/******************************************************************************
*
*   File Name:  spi3_to_spi4_arbiter.v
*     Version:  1.0
*        Date:  June 10, 2007
*
* Description:  The purpose of the spi3_to_spi4_arbiter.v file is to take 
*               the burst data from the four SPI-3 burst storage blocks and 
*               arbitrate writes into the SPI-4.2 Source Interface.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

`include "../../hdl/verilog/spi_pkg.v" 


/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi3_to_spi4_arbiter(
    Reset_n,
    BridgeClk,
    SPI4_Status,
    SPI4_AlmostFull_n,
    BrstRdEn,
    BrstRdChan,
    SPI4WrEn,
    BrstRdEOP,
    BrstValid,
    BrstEmpty,
    BrstAlmostEmpty,
    BrstPktCntEq0,
    BrstPktCntEq1
    );



    // System signals
    input                         Reset_n;
    input                         BridgeClk;
    input                  [7:0]  SPI4_Status;
    input                         SPI4_AlmostFull_n;
    
    // Burst Storage Outputs
    output                 [3:0]  BrstRdEn;
    output                 [1:0]  BrstRdChan;
    output                        SPI4WrEn;
    
    // Burst Storage Inputs    
    input                  [3:0]  BrstRdEOP;
    input                  [3:0]  BrstValid;
    input                  [3:0]  BrstEmpty;
    input                  [3:0]  BrstAlmostEmpty;
    input                  [3:0]  BrstPktCntEq0;
    input                  [3:0]  BrstPktCntEq1;


/*******************************************************************************
* Definition of Ports:
*******************************************************************************/
// System signals
//    Reset_n                   : System Level Reset (active low)
//    BridgeClk                 : Bridge Clock 
//
// SPI4 Status Input Signal
//    SPI4_Status[7:0]          : SPI4 Source Status signal
//
// SPI4 Source FIFO Almost Full Signal
//    SPI4_AlmostFull_n         : SPI4 Source FIFO Almost Full signal (active
//                                low)
//
// Burst Storage Outputs
//    BrstRdEn[3:0]             : Burst Storage read enable signal
//    BrstRdChan[1:0]           : Burst Storage read channel
//     
// Burst Storage Input Signals
//    BrstRdEOP                 : Burst Storage Read EOP * indicates if next
//                                read Burst Storage word has EOP
//    BrstRdValid               : Burst Storage Read Valid * indicates if Burst
//                                Storage has valid data
//    BrstAlmostEmpty           : Burst Storage is Almost Empty
//    BrstPktCntEq0[3:0]        : Burst Storage has zero EOPs (no complete
//                                packet)
//    BrstPktCntEq1[3:0]        : Burst Storage has only one EOP

  /*****************************************************************************
  * Parameter Declarations
  *****************************************************************************/
  parameter             SET_UP       = 2'b00,
                        TRANSFER     = 2'b01,
                        END_TRANSFER = 2'b10;

  /*****************************************************************************
  * Wire and Reg Declarations
  *****************************************************************************/
  //SPI4 Status signals
  reg                         [1:0] SPI4ChStatus [0:3];
  reg                         [1:0] Status;

  //Arbitration signals
  wire                        [1:0] Chan_int;
  wire                        [1:0] NxChan_int;
  reg                         [1:0] SchCh;  
  reg                         [1:0] NextChan;
  reg                               NextChValid;
  wire                              NextChValidEarly;
  reg                         [3:0] NextChDecode;
  reg                               NextChTaken;
  reg                         [3:0] RdEn;
  reg                         [1:0] CurrChan;
  reg                         [1:0] State;
  
  //Tx Burst Count signals
  reg                         [8:0] TxBurstLimit;
  reg                         [8:0] TxBurstCnt;
  reg                               TxBurstCntEn;
  reg                               TxBurstEnd;
  
  //SPI4 Source Write strobe signals
  reg                               WrEn;
  wire	                      [3:0] BrstRdEn;
  wire                        [1:0] BrstRdChan;
  wire                              SPI4WrEn;

  wire                       [1:0] SPI4ChStatus_Chan;
  wire                       [1:0] SPI4ChStatus_NxChan;

  /*****************************************************************************
  * Process to capture SPI4 Status for all 4 channels
  *****************************************************************************/
// pl4status block
  integer i;
  always @ (negedge Reset_n or posedge BridgeClk)
  begin
    if ( Reset_n == 0)
     begin
      SPI4ChStatus[0]  <= 2'b10;
      SPI4ChStatus[1]  <= 2'b10;
      SPI4ChStatus[2]  <= 2'b10;
      SPI4ChStatus[3]  <= 2'b10;
     end
    else
      for (i=0; i < 4; i = i+1)
       begin
        if (SPI4_AlmostFull_n == 0) 
	    SPI4ChStatus[i] <= 2'b10;
        else 
	    begin
	      SPI4ChStatus[0] <= SPI4_Status[1:0];
	      SPI4ChStatus[1] <= SPI4_Status[3:2];
	      SPI4ChStatus[2] <= SPI4_Status[5:4];
	      SPI4ChStatus[3] <= SPI4_Status[7:6];
	    end
       end
  end
  
  /*****************************************************************************
  * Process to find next eligible channel to transfer data from Burst Storage
  * to SPI4 Source FIFO
  * Channels are probed in round robin fashion
  * Channel is eligible for transfer if that channel's
  *     - Burst Storage is not Almost Empty OR
  *     - Has at least one EOP in Burst Storage (BrstPktCntEq0 = '0')
  *     - Burst Storage is not Empty
  *     - SPI4 Channel Status received is not SATISFIED ("10")
  *****************************************************************************/

  assign SPI4ChStatus_Chan   = SPI4ChStatus[Chan_int];
  assign SPI4ChStatus_NxChan = SPI4ChStatus[NxChan_int];

  // schaddr block
  always @ (negedge Reset_n or posedge BridgeClk)
   begin
    if (Reset_n == 0)
     begin
        SchCh        <= 2'b00;
        NextChan     <= 2'b00;
        NextChValid  <= 1'b0;
        NextChDecode <= 4'b0000;
     end
    else 
	if(NextChValid == 0 || NextChTaken == 1)
    	  begin      
	    SchCh       <= SchCh + 1;
            NextChan    <= Chan_int;
            NextChValid <= (((~BrstAlmostEmpty[Chan_int]) |
                           (~BrstPktCntEq0[Chan_int])) &
                           ((~BrstEmpty[Chan_int]) &
                           (~SPI4ChStatus_Chan[1])));

            for (i=0; i < 4; i = i+1)
             begin
              if(i == Chan_int)
               NextChDecode[i] <= 1;
              else
               NextChDecode[i] <= 0;
             end
        end
      else
            NextChValid <= (((~BrstAlmostEmpty[NxChan_int]) |
                           (~BrstPktCntEq0[NxChan_int])) &
                           (~BrstEmpty[NxChan_int]) &
                           (~SPI4ChStatus_NxChan[1]));     
  end

 
  assign Chan_int = SchCh;
  assign NxChan_int = NextChan;

  
  always @(NextChValid or State or TxBurstCntEn)
   begin
	if ((NextChValid == 1 && State == SET_UP) || (TxBurstCntEn == 0 && State == TRANSFER))
		NextChTaken <= 1; 
	else
		NextChTaken <= 0;
   end
  
  always @ (State or TxBurstEnd or BrstRdEOP)
   begin
	if ((State == TRANSFER && TxBurstEnd == 0) && (BrstRdEOP[CurrChan] == 0))
		TxBurstCntEn <= 1;
	else
		TxBurstCntEn <= 0;
   end

  assign NextChValidEarly = (((~BrstPktCntEq1[NxChan_int]) &
                            (~BrstPktCntEq0[NxChan_int])) |
                            (BrstPktCntEq1[NxChan_int] &
                            (~BrstRdEOP[NxChan_int])) | 
                            (BrstPktCntEq0[NxChan_int] & 
                            (~BrstAlmostEmpty[NxChan_int])));    


  /*****************************************************************************
  * FSM to generate read strobes for Burst Storage FIFOs.
  * SET_UP state      : This state is idle state and no read enables are 
  *                     generated until one of the channel is eligible for
  *                     transfer
  * TRANSFER state    : This is the TRANSFER state where read enable are
  *                     generated for eligible channel's Burst Storage FIFOs.
  *                     Transfer for a channel is ended when "MaxBurst1" or
  *                     "MaxBurst2" credits of data or an EOP is transfered
  * END_TRANSFER state: This ends reading from Burst Storage FIFOs.
  *****************************************************************************/
  // txfsm block
  always @ (negedge Reset_n or posedge BridgeClk)
  begin
    if (Reset_n == 0)
     begin
       State    <= SET_UP;
       RdEn     <= 4'b0000;
       CurrChan <= 2'b00;
     end
    else
      case (State)
        /**********************************************************************
        * State 0: SET_IP (Idle) state 
        **********************************************************************/
        SET_UP:
          if(NextChValid == 1)
	   begin
            State 	<= TRANSFER;
            RdEn 		<= NextChDecode;
            CurrChan 	<= NextChan;
	   end
          else
           begin  
	    State 	<= SET_UP;
            RdEn 	<= 4'b0000;
            CurrChan 	<= NextChan ;
	   end

        /**********************************************************************
        * State 1: TRANSFER State 
        **********************************************************************/        
        TRANSFER:
            if(TxBurstEnd == 1 || BrstRdEOP[CurrChan] == 1) 
	      begin
		if((NextChValid == 1) && (NextChValidEarly == 1))
		  begin
      	           State 		<= TRANSFER;
            	   RdEn 		<= NextChDecode;
	           CurrChan  	        <= NextChan;
		  end
                else 
	          begin
      	           State 		<= END_TRANSFER;
         	   RdEn 		<= 0;
	           CurrChan 	        <= NextChan;
      	          end
	      end
            else
	       State 		<= TRANSFER ;

        /**********************************************************************
        * State 2: END_TRANSFER State 
        **********************************************************************/
        END_TRANSFER:
	    begin
              State 	     <= SET_UP;
              RdEn 	     <= 4'b0000;
              CurrChan       <= NextChan;
	    end

        /**********************************************************************
        * NOP state 
        **********************************************************************/
        default: 
	   begin
             State 		<= State;
             RdEn 		<= RdEn;
             CurrChan 		<= CurrChan;
	   end
      endcase
    end

  /*****************************************************************************
  * Process to capture the status from SPI4 Source for Current Channel
  * SPI4 Source Status is Starving ("00") or Hungry("01")
  *****************************************************************************/
  // statusreg block
  always @ (negedge Reset_n or posedge BridgeClk)
   begin
     if(Reset_n == 0)
       Status <= 2'b10;
     else
       Status <= SPI4ChStatus[NextChan];
   end

  /*****************************************************************************
  * Process to count number of SPI4 Words (8 bytes) that have
  * been transferred from channel's Burst Storage to SPI4 Source FIFOs
  * MaxBurst1 and MaxBurst2 are indicated in number or credits (16 Bytes)
  * MaxBurst1 should be > 1 (16 bytes)
  *****************************************************************************/
  // maxburstcnt block
  always @(negedge Reset_n or posedge BridgeClk)
    begin
      if(Reset_n == 0)
       begin
        TxBurstLimit 	<= 0;
        TxBurstCnt 	<= 0;
        TxBurstEnd 	<= 0;
       end
      else 
       begin
	if (TxBurstCntEn == 0)
         begin
          if(Status == 2'b00)
            TxBurstLimit <= (2*(`MaxBurst1));
          else
            TxBurstLimit <= (2*(`MaxBurst2));
         end  

        if(TxBurstCntEn == 0)
          TxBurstCnt <= 0;
        else
          TxBurstCnt <= TxBurstCnt + 1 ;

        if(TxBurstCntEn == 0)
          TxBurstEnd <= 0;
        else if(TxBurstCnt == (TxBurstLimit-2))
          TxBurstEnd <= 1;
       end
    end
    
  /*****************************************************************************
  * Process to generate Write strobe for SPI4 Source FIFO
  *****************************************************************************/
  // spi4wr_strobe block
  always @ (negedge Reset_n or posedge BridgeClk)
   begin
     if(Reset_n == 0)
       WrEn <= 0;
     else
       WrEn <= RdEn[CurrChan];
   end
    
  /*****************************************************************************
  * Output Signal Assignments
  *****************************************************************************/

    assign BrstRdEn   = RdEn;
    assign BrstRdChan = CurrChan;
    assign SPI4WrEn   = WrEn;
   
endmodule
