/******************************************************************************
*
*   File Name:  spi3_to_spi4_read.v
*     Version:  2.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi3_to_spi4_read.v file is to provide 
*               the SPI-3 core read logic. This logic will read data from one
*               SPI-3 interface as long as it is not empty, and the SPI3
*               burst storage is not full.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/
`include "../../hdl/verilog/spi_pkg.v"
// `include "../../../hdl/verilog/spi_pkg.v" 

/******************************************************************************
* Module Declaration
*******************************************************************************/

module spi3_to_spi4_read (
    Reset_n,
    BridgeClk,
    SPI3_FFAddr,
    SPI3_FFData,
    SPI3_FFMod,
    SPI3_FFSOP,
    SPI3_FFEOP,
    SPI3_FFErr,
    SPI3_SRC_RDY,
	 SPI3_DST_RDY,
    SPI3_Full_n,
    SPI3_Addr,
    SPI3_Data,
    SPI3_REM,
    SPI3_SOP,
    SPI3_EOP,
    SPI3_Err,
    SPI3_Valid
    );

    // System signals
    input                       Reset_n;
    input                       BridgeClk;

    // SPI3 Core Interface Input Signals
    input               [7:0]   SPI3_FFAddr;
    input              [31:0]   SPI3_FFData;
	 input					[1:0]	  SPI3_FFMod;
    input                       SPI3_FFSOP;
    input                       SPI3_FFEOP;
    input                       SPI3_FFErr;
    input                       SPI3_SRC_RDY;

    // SPI3 Core Interface Output Signals
    output                      SPI3_DST_RDY;
    
    // SPI3 Burst Storage Interface Input Signals
    input                       SPI3_Full_n;

    // SPI3 Burst Storage Interface Output Signals
    output              [7:0]   SPI3_Addr;
    output             [31:0]   SPI3_Data;
	 output				  [1:0]    SPI3_REM;		
    output                      SPI3_SOP;
    output                      SPI3_EOP;
    output                      SPI3_Err;
    output                      SPI3_Valid;

/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n                : System Level Reset (active low)
*    BridgeClk              : SPI3 Interface Clock 

* SPI3 Core Interface Input Signals
*    SPI3_FFAddr           : SPI3 Interface Address
*    SPI3_FFData           : SPI3 Interface Data
*    SPI3_FFMod            : SPI3 Interface MOD
*    SPI3_FFSOP            : SPI3 Interface Start of Packet 
*    SPI3_FFEOP            : SPI3 Interface End of Packet 
*    SPI3_FFErr            : SPI3 Interface Error 
*    SPI3_SRC_RDY          : SPI3 Interface is Ready

* SPI3 Core Interface Output Signals
*    SPI3_DST_RDY          : SPI3 Interface Enable

* SPI3 Burst Storage Interface Input Signals
*    SPI3_Full_n           : Burst Logic Full (active low) 

* SPI3 Burst Storage Interface Output Signals
*    SPI3_Addr             : Burst Logic Address 
*    SPI3_Data             : Burst Logic Data 
*    SPI3_REM              : Burst Logic MOD 
*    SPI3_SOP              : Burst Logic Start of Packet 
*    SPI3_EOP              : Burst Logic End of Packet  
*    SPI3_Err              : Burst Logic Error  
*    SPI3_Valid            : Burst Logic Data is Valid  
*/
  /*****************************************************************************
  * Constant Declarations
  *****************************************************************************/

  /*****************************************************************************
  * State Machine Variable and Type Declarations
  * StateType: 
  *    IDLE             - Reset and idle state.
  *    REQUEST          - A data transfer request. This indicates that the
  *                       loopback is enabled, the Receive FIFO has data, and
  *                       the Transmit FIFO can accept data.
  *    TRANSFER         - Data is transferred in this state. Remain in this
  *                       state until the loopback is disabled,
  *                       the Receive FIFO
  *                       doesn't have any more data, or the
  *                       Transmit FIFO cannot
  *                       accept any more data.
  *    ONE_TRANSFER     - Indicates that the Receive FIFO has only one word
  *                       available for reading.
  *
  * Variables:
  *    State   - holds current state of state machine
  *****************************************************************************/

  parameter		IDLE         = 2'b00, 
			REQUEST      = 2'b01, 
                        TRANSFER     = 2'b10, 
                        ONE_TRANSFER = 2'b11;
  reg 	    [1:0] 	State;

  /*****************************************************************************
  * Signal Declarations
  * OneWordLeft - Indicates that only one word is left in the FIFO, and a 
  *               single read should be performed
  *****************************************************************************/

  reg                   OneWordLeft;
  reg                   SPI3_SRC_r;
  reg                   SPI3_DST_RDY;
  reg						   SPI3_Valid;

  /*****************************************************************************
  * Drive out the inputs from the Receive FIFO.
  *****************************************************************************/
  assign SPI3_Data   = SPI3_FFData;
  assign SPI3_Addr   = SPI3_FFAddr;
  assign SPI3_SOP    = SPI3_FFSOP;
  assign SPI3_EOP    = SPI3_FFEOP;
  assign SPI3_Err    = SPI3_FFErr;
  assign SPI3_REM		= SPI3_FFMod;


  /*****************************************************************************
  * Determine if only one word is left in the FIFO. This is true only if 
  * Empty_n was asserted on the previous cycle, and AlmostEmpty_n is not 
  * asserted on the current cycle.
  *****************************************************************************/
//  // reg_empty block
//  always @ (negedge Reset_n or posedge BridgeClk)
//   begin
//    if (Reset_n == 0)
//     begin
//      SPI3_FFEmpty_n_r       <= 0;
//      SPI3_FFAlmostEmpty_n_r <= 0;
//     end
//    else
//     begin
//      SPI3_FFEmpty_n_r       <= SPI3_FFEmpty_n ;
//      SPI3_FFAlmostEmpty_n_r <= SPI3_FFAlmostEmpty_n ;
//     end
//   end
//
//  /*****************************************************************************
//  * In order for only one word left to be true, then not AE_n and E_n needs
//  * to be true for at least two cycles. This prevents flagging an incorrect
//  * one word left indication as the FIFO is transitioning from E to AE.
//  *****************************************************************************/
//
//  // one_left block
//  always @ (negedge Reset_n or posedge BridgeClk)
//   begin
//      if (Reset_n == 0)
//        OneWordLeft <= # `TFF  0;
//      else
//        OneWordLeft <= ((~SPI3_FFAlmostEmpty_n & SPI3_FFEmpty_n) &
//                       (~SPI3_FFAlmostEmpty_n_r & SPI3_FFEmpty_n_r));
//   end

  //****************************************************************************
  // State Machine Process
  //****************************************************************************

  // state_machine block
  always @ (negedge Reset_n or posedge BridgeClk )
   begin
    if(Reset_n == 0)
     begin
      State         <= IDLE;
      SPI3_Valid <= 1'b0 ;
		SPI3_DST_RDY <= 1'b0;
     end
    else
      case (State)
        /**********************************************************************
        * State 0: IDLE 
        **********************************************************************/
        IDLE:
          if ((SPI3_SRC_RDY == 1) && (SPI3_Full_n == 1))
           begin
            State         <= TRANSFER;
            SPI3_Valid <= 1'b1 ;
				SPI3_DST_RDY <= 1'b1;
           end
          else
           begin
				State         <= IDLE;
            SPI3_Valid <= 1'b0 ;
				SPI3_DST_RDY <= 1'b0;
	   end

//        /**********************************************************************
//        * State 1: REQUEST
//        **********************************************************************/
//        REQUEST:
//         begin
//          State         <= TRANSFER;
//          SPI3_FFRdEn_n <= 0;
//         end

        /**********************************************************************
        * State 2: TRANSFER
        **********************************************************************/
        TRANSFER:
          if ((SPI3_SRC_RDY == 1) && (SPI3_Full_n == 1))
           begin
            State         <= TRANSFER;
            SPI3_Valid <= 1'b1 ;
				SPI3_DST_RDY <= 1'b1;
           end
          else
           begin
            State         <= IDLE;
            SPI3_Valid <= 1'b0 ;
				SPI3_DST_RDY <= 1'b0;
           end

        /**********************************************************************
        * State 3: ONE_TRANSFER
        **********************************************************************/
//        ONE_TRANSFER:
//         begin
//          State         <= IDLE;
//          SPI3_FFRdEn_n <= 1;
//         end
 
       /**********************************************************************
        * Error and Default States
        **********************************************************************/
        default:
         begin
          State         <= IDLE;
          SPI3_Valid <= 1'b0 ;
			 SPI3_DST_RDY <= 1'b0;
         end
      endcase
  end

endmodule
