/******************************************************************************
*
*   File Name:  spi4_to_spi3_flow_control.v
*     Version:  1.0
*        Date:  June 11, 2004
*
* Description:  The purpose of the spi4_to_spi3_flow_control.v file is to 
*               drive the SPI4.2 Core Status Interface with the proper flow
*               control information depending on signals received from the
*               SPI-3 cores and the burst storage FIFOs.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/
`timescale 1 ns/1 ps
/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi4_to_spi3_flow_control(
    Reset_n,
    SPI4_Clk,
    SPI3_0_DST_RDY,
    SPI3_1_DST_RDY,
    SPI3_2_DST_RDY,
    SPI3_3_DST_RDY,
    SPI3_0_Brst_HalfFull,
    SPI3_1_Brst_HalfFull,
    SPI3_2_Brst_HalfFull,
    SPI3_3_Brst_HalfFull,
    SPI4_StatWrEn_n,
    SPI4_Stat
    );

    // System signals
    input                   Reset_n;
    input                   SPI4_Clk;

    // SPI3 Core Status Input Signals
    input                   SPI3_0_DST_RDY;
    input                   SPI3_1_DST_RDY;
    input                   SPI3_2_DST_RDY;
    input                   SPI3_3_DST_RDY;
    
    // Burst Storage Status Signals
    input                   SPI3_0_Brst_HalfFull;
    input                   SPI3_1_Brst_HalfFull;
    input                   SPI3_2_Brst_HalfFull;
    input                   SPI3_3_Brst_HalfFull;
 
    // SPI4 Status Output Signals
    output                  SPI4_StatWrEn_n;
    output           [7:0]  SPI4_Stat;

    reg              [7:0]  SPI4_Stat;


/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n                   : System Level Reset (active low)
*    SPI4_Clk                  : SPI4 Interface Clock 
*
* SPI3 Core Status Input Signals
*    SPI3_0_DST_RDY      		 : SPI3 0 is Ready
*    SPI3_1_DST_RDY     	  	 : SPI3 1 is Ready
*    SPI3_2_DST_RDY       		 : SPI3 2 is Ready
*    SPI3_3_DST_RDY       		 : SPI3 3 is Ready
*
* Burst Storage Status Signals
*    SPI3_0_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
*    SPI3_1_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
*    SPI3_2_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
*    SPI3_3_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
*
* SPI4 Status Output Signals
*    SPI4_StatWrEn_n           : SPI4 Status Write Enable (active low)
*    SPI4_Stat                 : SPI4 Status - 2-bits per channel
*                                (7:6) = Channel 3, (1:0) = Channel 0
*******************************************************************************/


always @ (posedge SPI4_Clk or negedge Reset_n)
  begin
    if (Reset_n == 0)
      SPI4_Stat             <= 8'b10101010;
    // "00" = starving, "10" = satisfied, "01" = hungry, "11" = framing
    // If SPI 3 source core is Almost Full or burst FIFO is half full,
    // then send satisified on the Full channel. Else, send starving.

    else 
    begin
      if (SPI3_0_DST_RDY == 0 || SPI3_0_Brst_HalfFull == 1)
        SPI4_Stat[1:0] <= 2'b10;
      else
        SPI4_Stat[1:0] <= 2'b00;
 
      if (SPI3_1_DST_RDY == 0 || SPI3_1_Brst_HalfFull == 1)
        SPI4_Stat[3:2] <= 2'b10;
      else
        SPI4_Stat[3:2] <= 2'b00;
      
      if (SPI3_2_DST_RDY == 0 || SPI3_2_Brst_HalfFull == 1)
        SPI4_Stat[5:4] <= 2'b10;
      else
        SPI4_Stat[5:4] <= 2'b00;
      
      if (SPI3_3_DST_RDY == 0 || SPI3_3_Brst_HalfFull == 1)
        SPI4_Stat[7:6] <= 2'b10;
      else
        SPI4_Stat[7:6] <= 2'b00;
    end
  end
  
  assign SPI4_StatWrEn_n = 0;

endmodule