/******************************************************************************
*
*   File Name:  spi4_to_spi3_read.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi4_to_spi3_read.v file is to provide 
*               the SPI/4.2 core read logic. This logic will read data from 
*               the SPI/4.2 Core as long as the SPI4.2 Core is not empty.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

/******************************************************************************
* Module Declaration
*******************************************************************************/

module spi4_to_spi3_read (
	Reset_n,
	SPI4_Clk,
	SPI4_FFAddr,
	SPI4_FFData,
	SPI4_FFMod,
	SPI4_FFSOP,
	SPI4_FFEOP,
	SPI4_FFErr,
	SPI4_FFValid,
	SPI4_FFPayloadErr,
	SPI4_FFDIP4Err,
	SPI4_FFEmpty_n,
	SPI4_FFAlmostEmpty_n,
	SPI4_FFRdEn_n,
    	SPI4_Addr,
	SPI4_Data,
	SPI4_Mod,
	SPI4_SOP,
	SPI4_EOP,
	SPI4_Err,
	SPI4_Valid
    );

 	// System signals
	input              Reset_n;
	input              SPI4_Clk;

	// SPI4 Core Interface Input Signals
	input       [7:0]  SPI4_FFAddr;
	input      [63:0]  SPI4_FFData;
	input       [2:0]  SPI4_FFMod;
	input              SPI4_FFSOP;
	input              SPI4_FFEOP;
	input              SPI4_FFErr;
	input              SPI4_FFValid;
	input              SPI4_FFPayloadErr;
	input              SPI4_FFDIP4Err;
	input              SPI4_FFEmpty_n;
	input              SPI4_FFAlmostEmpty_n;

	// SPI4 Core Interface Output Signals
	output             SPI4_FFRdEn_n;
    
	// SPI4 Write Logic Interface Output Signals
	output      [7:0]  SPI4_Addr;
	output     [63:0]  SPI4_Data;
	output      [2:0]  SPI4_Mod;
	output             SPI4_SOP;
	output             SPI4_EOP;
	output             SPI4_Err;
	output             SPI4_Valid;
     
/*******************************************************************************
* Definition of Ports:
*******************************************************************************/

//System signals
//    Reset_n               : System Level Reset (active low)
//    SPI4_Clk              : SPI4 Interface Clock 

// SPI4 Core Interface Input Signals
//    SPI4_FFAddr           : SPI4 Interface Address
//    SPI4_FFData           : SPI4 Interface Data
//    SPI4_FFMod            : SPI4 Interface MOD
//    SPI4_FFSOP            : SPI4 Interface Start of Packet 
//    SPI4_FFEOP            : SPI4 Interface End of Packet 
//    SPI4_FFErr            : SPI4 Interface Error 
//    SPI4_FFValid          : SPI4 Interface Data is Valid 
//    SPI4_FFPayloadErr     : SPI4 Interface Payload Error (current data was
//                            not preceded by a payload control word
//    SPI4_FFDIP4Err        : SPI4 Interface DIP/4 Error was detected on current
//                            end of burst (either EOP or N/credit boundary)
//    SPI4_FFEmpty_n        : SPI4 Interface is Empty (active low) 
//    SPI4_FFAlmostEmpty_n  : SPI4 Interface is Almost Empty (active low)

// SPI4 Core Interface Output Signals
//    SPI4_FFRdEn_n         : SPI4 Interface Read Enable (active low)

// SPI4 Write Logic Interface Output Signals
//    SPI4_Addr             : Write Logic Address 
//    SPI4_Data             : Write Logic Data 
//    SPI4_Mod              : Write Logic MOD 
//    SPI4_SOP              : Write Logic Start of Packet 
//    SPI4_EOP              : Write Logic End of Packet  
//    SPI4_Err              : Write Logic Error  
//    SPI4_Valid            : Write Logic Data is Valid  

/*******************************************************************************
* spi4_to_spi3_read Architecture
*******************************************************************************/

  /*****************************************************************************
  * Constant Declarations
  *****************************************************************************/

  /*****************************************************************************
  * State Machine Variable and Type Declarations
  * StateType: 
  *    IDLE             / Reset and idle state.
  *    REQUEST          / A data transfer request. This indicates that the
  *                       loopback is enabled, the Receive FIFO has data, and
  *                       the Transmit FIFO can accept data.
  *    TRANSFER         / Data is transferred in this state. Remain in this
  *                       state until the loopback is disabled,
  *                       the Receive FIFO
  *                       doesn't have any more data, or the
  *                       Transmit FIFO cannot
  *                       accept any more data.
  *    ONE_TRANSFER     / Indicates that the Receive FIFO has only one word
  *                       available for reading.
  *
  * Variables:
  *    State   / holds current state of state machine
  ****************************************************************************/

  parameter            IDLE         = 2'b00,
                       REQUEST      = 2'b01, 
                       TRANSFER     = 2'b10, 
                       ONE_TRANSFER = 2'b11;
  reg           [1:0]  State;

  /****************************************************************************
  * Signal Declarations
  * OneWordLeft / Indicates that only one word is left in the FIFO, and a 
  *              single read should be performed
  ****************************************************************************/
  reg	               OneWordLeft;
  reg                  SPI4_FFEmpty_n_r;
  reg                  SPI4_FFAlmostEmpty_n_r;
  wire                 Reset_n;
  wire                 SPI4_Clk;
  wire          [7:0]  SPI4_FFAddr;
  wire         [63:0]  SPI4_FFData;
  wire          [2:0]  SPI4_FFMod;
  wire                 SPI4_FFSOP;
  wire                 SPI4_FFEOP;
  wire                 SPI4_FFErr;
  wire                 SPI4_FFValid;
  wire                 SPI4_FFPayloadErr;
  wire                 SPI4_FFDIP4Err;
  wire                 SPI4_FFEmpty_n;
  wire                 SPI4_FFAlmostEmpty_n;
  reg                  SPI4_FFRdEn_n;
  wire          [7:0]  SPI4_Addr;
  wire         [63:0]  SPI4_Data;
  wire          [2:0]  SPI4_Mod;
  wire                 SPI4_SOP;
  wire                 SPI4_EOP;
  wire                 SPI4_Err;
  wire                 SPI4_Valid;
	

  /****************************************************************************
  * Drive out the inputs from the Receive FIFO.
  ****************************************************************************/
  assign 	SPI4_Data   = SPI4_FFData;
  assign	SPI4_Addr   = SPI4_FFAddr;
  assign	SPI4_Mod    = SPI4_FFMod;
  assign	SPI4_SOP    = SPI4_FFSOP;
  assign	SPI4_EOP    = SPI4_FFEOP;
  assign	SPI4_Err    = SPI4_FFErr;
  assign	SPI4_Valid  = SPI4_FFValid;

  /****************************************************************************
  * Determine if only one word is left in the FIFO. This is true only if 
  * Empty_n was asserted on the previous cycle, and AlmostEmpty_n is not 
  * asserted on the current cycle.
  ****************************************************************************/
  
  // reg_empty block
  always @ (posedge SPI4_Clk or negedge Reset_n)
    begin
	if (Reset_n == 0)
	  begin
	    SPI4_FFEmpty_n_r       <= 0;
     	    SPI4_FFAlmostEmpty_n_r <= 0;
	  end
	else
	  begin
	    SPI4_FFEmpty_n_r       <= SPI4_FFEmpty_n ;
      	    SPI4_FFAlmostEmpty_n_r <= SPI4_FFAlmostEmpty_n ;
	  end
    end

  // In order for only one word left to be true, then not AE_n and E_n needs
  // to be true for at least two cycles. This prevents flagging an incorrect
  // one word left indication as the FIFO is transitioning from E to AE.
  always @ (posedge SPI4_Clk or negedge Reset_n) 
    begin
	if(Reset_n == 0)
		OneWordLeft	   <= 0;
	else
		OneWordLeft <= ((~SPI4_FFAlmostEmpty_n & SPI4_FFEmpty_n) &
                      (~ SPI4_FFAlmostEmpty_n_r & SPI4_FFEmpty_n_r));
    end

  /****************************************************************************
  * State Machine Process
  ****************************************************************************/

  //state_machine block
  always @ (posedge SPI4_Clk or negedge Reset_n)
    begin
	if (Reset_n == 0)
	  begin
	      State 			<= IDLE;
	      SPI4_FFRdEn_n 		<= 1;
	  end
	else
	  case (State)
        /**********************************************************************
	  * State 0: IDLE 
        **********************************************************************/          
        IDLE:
          //If the PL4 core is not almost empty, start reading
          if (SPI4_FFAlmostEmpty_n == 1)
	   begin 	
            State         		<= REQUEST ;
            SPI4_FFRdEn_n 		<= 0;
           end
          //If there is only one word left, only transfer one word out
          //of the PL4 core
          else if (OneWordLeft == 1)
           begin
            State         		<= ONE_TRANSFER ;
            SPI4_FFRdEn_n 		<= 0;
           end
          else
           begin
            State         		<= IDLE ;
            SPI4_FFRdEn_n 		<= 1;
           end
        /**********************************************************************
        * State 1: REQUEST
        **********************************************************************/
        REQUEST:
	  begin
            State         		<= TRANSFER ;
            SPI4_FFRdEn_n 		<= 0;
          end
        /**********************************************************************
        * State 2: TRANSFER
        **********************************************************************/
        TRANSFER:
          if (SPI4_FFAlmostEmpty_n == 1)
           begin
            State         		<= TRANSFER ;
            SPI4_FFRdEn_n 		<= 0;
           end
          else
           begin
            State 		      <= IDLE ;
            SPI4_FFRdEn_n 		<= 1;
           end
        /**********************************************************************
        * State 3: ONE_TRANSFER
        **********************************************************************/
        ONE_TRANSFER:
         begin
          State		         	<= IDLE ;
          SPI4_FFRdEn_n 		<= 1;
         end
        /**********************************************************************
        * Error and Default States
        **********************************************************************/
        default:
         begin
          State         		<= IDLE ;
          SPI4_FFRdEn_n 		<= 1;
	 end
      endcase
    end

endmodule
