/******************************************************************************
*
*   File Name:  spi_clk_startup.v
*     Version:  1.0
*        Date:  June 11, 2007
*
* Description:  The purpose of the spi_clk_startup.v file is to set the
*               proper startup sequence for the DCMs. This will allow proper
*               initialization of the clocking scheme for the entire system.
*
*      Author:  Zhe Xia
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
*******************************************************************************/

`include "../../hdl/verilog/spi_pkg.v"

/*******************************************************************************
* Module Declaration
*******************************************************************************/

module spi_clk_startup(
	Reset_n,
	SPI_Clk,
	SPI4_DcmReset_RDClk,
	SPI4_Locked_RDClk,
	SPI4_DcmReset_TDClk,
	SPI4_Locked_TDClk,
	SPI4_DcmReset_TSClk,
	SPI4_Locked_TSClk,
	PL4Rst_n,
	SnkStatClk,
	SrcStatClk,
	CalProg,
	SnkCalProgDone,
	SrcCalProgDone,
	SnkEn,
	SrcEn,
	SnkFFClk,
	PhaseAlignRequest,
	PhaseAlignComplete,
	SnkOof   
	);

	
	input			Reset_n;
	input			SPI_Clk;

    	// SPI4.2 Sink DCM Status
	output		SPI4_DcmReset_RDClk;
	input			SPI4_Locked_RDClk;
	
	// SPI4.2 Source DCM Status
	output		SPI4_DcmReset_TDClk;
	input    		SPI4_Locked_TDClk;
	output		SPI4_DcmReset_TSClk;
	input			SPI4_Locked_TSClk;

	output		PL4Rst_n;
    
	// Calendar Programming
	input			SnkStatClk;
	input			SrcStatClk;
	output		CalProg;
    	input			SnkCalProgDone;
    	input			SrcCalProgDone;
    
	// Sink and Source Core enables
	output		SnkEn;
	output		SrcEn;
    
	// Dynamic phase aligning signals
	input			SnkFFClk;
	output		PhaseAlignRequest;
	input			PhaseAlignComplete;
	input			SnkOof;
    
/*******************************************************************************
* Definition of Ports:
********************************************************************************
* System signals
*    Reset_n                 : Input Reset (active low)
*    SPI_Clk                 : SPI Interface Clock 
*    SPI4_DcmReset_RDClk     : RDClk DCM Reset (active high)
*    SPI4_Locked_RDClk       : RDClk DCM Locked signal (active high)
*    SPI4_DcmReset_TDClk     : TDClk DCM Reset (active high)
*    SPI4_Locked_TDClk       : TDClk DCM Locked signal (active high)
*    SPI4_DcmReset_TSClk     : TSClk DCM Reset (active high)
*    SPI4_Locked_TSClk       : TSClk DCM Locked signal (active high)
*    PL4Rst_n                : System Level Reset indicating all DCMs are
*                              locked
*    SnkStatClk              : Clock used for programming the Snk calendar
*    SrcStatClk              : Clock used for programming the Src calendar
*    SnkCalProgDone          : Signal indicating the Snk calendar programming
*                              logic has completed programming
*    SrcCalProgDone          : Signal indicating the Src calendar programming
*                              logic has completed programming
*    SnkEn                   : Enable signal for the SPI-4.2 sink core
*    SrcEn                   : Enable signal for the SPI-4.2 source core
*    SnkFFClk                : Clock used for talking to the user side of the
*                              SPI-4.2 sink core
*    PhaseAlignRequest       : Signal indicating that the SPI-4.2 sink core
*                              should re-align the data with the clock
*    PhaseAlignComplete      : Signal indicated the SPI-4.2 sink core has
*                              complete re-aligning the data with the clock
*    SnkOof                  : Signal indicating the data and clock are mis-
*                              aligned in the SPI-4.2 sink core
*******************************************************************************/

  /*****************************************************************************
  * Constant Declarations
  *****************************************************************************/

  /*****************************************************************************
  * Signal Declarations
  *****************************************************************************/

	parameter		IDLE = 3'b000,
      	            TDCLK_RST = 3'b001,
            	      TDCLK_LCK = 3'b010,
                  	RDCLK_RST = 3'b011,
	                  RDCLK_LCK = 3'b100,
      	            TSCLK_RST = 3'b101,
            	      TSCLK_LCK = 3'b110,
                  	RELEASE_RST = 3'b111;
                 
	parameter		PA_IDLE  = 2'b00,
          	            PA_REQ   = 2'b01,
            	      PA_CMPLT = 2'b10;
	
	reg			[2:0] state;
 	reg			CountClear;
	reg			[8:0] Count;
	reg			[1:0] state_snk;
	reg		      [6:0] wait_count;
	reg			PL4Rst_n_int;
	reg			SrcEn_i;
	reg			SnkEn_i;
	wire			Reset_n;
	wire			SPI_Clk;
	reg			SPI4_DcmReset_RDClk;
	wire			SPI4_Locked_RDClk;
	reg			SPI4_DcmReset_TDClk;
	wire    		SPI4_Locked_TDClk;
	reg			SPI4_DcmReset_TSClk;
	wire			SPI4_Locked_TSClk;
	wire			PL4Rst_n;
   	wire			SnkStatClk;
	wire			SrcStatClk;
	wire			CalProg;
    	wire			SnkCalProgDone;
    	wire			SrcCalProgDone;
    	wire			SnkEn;
	wire			SrcEn;
    	wire			SnkFFClk;
	reg			PhaseAlignRequest;
	wire			PhaseAlignComplete;
	wire			SnkOof;

// count block
always @ (posedge SPI_Clk or negedge Reset_n)
begin
	if (Reset_n == 0)
		Count <= 0;
	else if (CountClear == 1)
		Count <= 0;
	else
		Count <= Count + 1;

end

// rst_fsm block
always @ (posedge SPI_Clk or negedge Reset_n)
begin
	if (Reset_n == 0)
	begin
		state <= IDLE;
		SPI4_DcmReset_TDClk <= 0;
		SPI4_DcmReset_RDClk <= 0;
		SPI4_DcmReset_TSClk <= 0;
		CountClear <= 1;
		PL4Rst_n_int <= 0;
	end
	else
		case(state)
			IDLE: begin
				state <= TDCLK_RST;
				SPI4_DcmReset_TDClk <= 1;
				SPI4_DcmReset_RDClk <= 0;
				SPI4_DcmReset_TSClk <= 0;
			      CountClear          <= 1;
			      PL4Rst_n_int        <= 0;
				end
      		TDCLK_RST:  begin
					// Expand DCM Reset pulse duration to 8 clock cycles
				        if (Count == 8) 
					  begin
					      state               <= TDCLK_LCK;
					      SPI4_DcmReset_TDClk <= 0;
					  end
				        else
					  begin				          
						state               <= TDCLK_RST;
				      	SPI4_DcmReset_TDClk <= 1;
				        end
				        SPI4_DcmReset_RDClk <= 0;
				        SPI4_DcmReset_TSClk <= 0;
				        CountClear          <= 0;
				        PL4Rst_n_int        <= 0;
				      end

			TDCLK_LCK:  begin
					// If TDCLK DCM is locked, then go to next state
					  if (SPI4_Locked_TDClk == 1)
					  begin	
				          state               <= RDCLK_RST;
				          SPI4_DcmReset_RDClk <= 1;
				          SPI4_DcmReset_TDClk <= 0;
				          CountClear          <= 1;
					  end
			          // If counter has wrapper all the way around (256 cycles), then reset
			          // the DCM again and go back to TDCLK_RST.
				        else if (Count == 8)
					  begin
				            state               <= TDCLK_RST;
				            SPI4_DcmReset_TDClk <= 1;
				            SPI4_DcmReset_RDClk <= 0;
				            CountClear          <= 1;
					  end
			        // Else wait for the DCM to lock or the counter to expire
				        else
					  begin
						state               <= TDCLK_LCK;
						SPI4_DcmReset_TDClk <= 0;
						SPI4_DcmReset_RDClk <= 0;
						CountClear          <= 0;
					  end
				  SPI4_DcmReset_TSClk <= 0;
				  PL4Rst_n_int        <= 0;
				  end

			RDCLK_RST: begin
				     // Expand DCM Reset pulse duration to 8 clock cycles
			          if (Count == 8)
				      begin
				          state               <= RDCLK_LCK;
				          SPI4_DcmReset_RDClk <= 0;
				      end
			          else
				      begin
				          state               <= RDCLK_RST;
				          SPI4_DcmReset_RDClk <= 1;
			            end
			         SPI4_DcmReset_TDClk <= 0;
			         SPI4_DcmReset_TSClk <= 0;
			         CountClear          <= 0;
			         PL4Rst_n_int        <= 0;
				   end

			RDCLK_LCK: begin
			           // If RDCLK DCM is locked, then go to next state
			           if (SPI4_Locked_RDClk == 1) 
				     	begin
				          state               <= TSCLK_RST;
				          SPI4_DcmReset_RDClk <= 0;
				          SPI4_DcmReset_TSClk <= 1;
				          CountClear          <= 1;
				     	end
			           // If counter has wrapper all the way around (256 cycles), then reset
			           // the DCM again and go back to RDCLK_RST.
			          else if (Count == 8)
				    	begin
				          state               <= RDCLK_RST;
				          SPI4_DcmReset_RDClk <= 1;
				          SPI4_DcmReset_TSClk <= 0;
				          CountClear          <= 1;
				     	end
			           // Else wait for the DCM to lock or the counter to expire
				     else
				     	begin
				          state               <= RDCLK_LCK;	
				          SPI4_DcmReset_RDClk <= 0;
				          SPI4_DcmReset_TSClk <= 0;
				          CountClear          <= 0;
			          	end
				    SPI4_DcmReset_TDClk <= 0;
				    PL4Rst_n_int        <= 0;
				    end

				TSCLK_RST: begin
			        // Expand DCM Reset pulse duration to 8 clock cycles
			        if (Count == 8)
					begin
				          state               <= TSCLK_LCK;
				          SPI4_DcmReset_TSClk <= 0;
					end
			        else
					begin
				          state               <= TSCLK_RST;
				          SPI4_DcmReset_TSClk <= 1;
					end
			        SPI4_DcmReset_TDClk <= 0;
			        SPI4_DcmReset_RDClk <= 0;
			        CountClear          <= 0;
			        PL4Rst_n_int        <= 0;
				end
				
				TSCLK_LCK: begin
			        // If TSCLK DCM is locked, then go to next state
			        if (SPI4_Locked_TSClk == 1)
					begin
				          state               <= RELEASE_RST;
				          SPI4_DcmReset_TSClk <= 0;
				          CountClear          <= 1;
					end
			        // If counter has wrapper all the way around (256 cycles), then reset
			        // the DCM again and go back to TSCLK_RST.
			        else if (Count == 8)
					begin
				          state               <= TSCLK_RST;
				          SPI4_DcmReset_TSClk <= 1;
				          CountClear          <= 1;
					end
			        // Else wait for the DCM to lock or the counter to expire
			        else
					begin
				          state               <= TSCLK_LCK;
				          SPI4_DcmReset_TSClk <= 0;
				          CountClear          <= 0;
					end
			        SPI4_DcmReset_TDClk <= 0;
			        SPI4_DcmReset_RDClk <= 0;
			        PL4Rst_n_int        <= 0;
				end

			RELEASE_RST: begin
			        SPI4_DcmReset_TDClk <= 0;
			        SPI4_DcmReset_RDClk <= 0;
			        SPI4_DcmReset_TSClk <= 0;
			        CountClear          <= 1;      
			        PL4Rst_n_int        <= 1;
			end
      
			default:begin
			        state               <= IDLE;
			        SPI4_DcmReset_TDClk <= 0;
			        SPI4_DcmReset_RDClk <= 0;
			        SPI4_DcmReset_TSClk <= 0 ;
			        CountClear          <= 1;
			        PL4Rst_n_int        <= 0;
			end

    		endcase
end

// src_en block
always @(posedge SrcStatClk or negedge PL4Rst_n_int)
begin
	if (PL4Rst_n_int == 0)
	    SrcEn_i <= 0;    
	else
	    SrcEn_i <= SrcCalProgDone;    
end

// snk_en block
always @(posedge SnkStatClk or negedge PL4Rst_n_int)
begin
	if (PL4Rst_n_int == 0)
	    SnkEn_i <= 0;
	else
	    SnkEn_i <= SnkCalProgDone;
end

// snk_fsm block
always @ (posedge SnkFFClk or negedge Reset_n)
begin
	if (Reset_n == 0)
	  begin
	    PhaseAlignRequest  <= 0;
	    state_snk          <= PA_IDLE;
	    wait_count         <= 0;
        end
	else
         case (state_snk)
  	      // PA_IDLE State : wait for completion of DCM resets
		PA_IDLE: 
			if (SnkOof==1 && SnkEn_i==1)
			  begin
		          PhaseAlignRequest  <= 0;
		          state_snk          <= PA_REQ;
		          wait_count         <= 0;
			  end
		      else
			  begin
		          PhaseAlignRequest  <= 0;
		          state_snk          <= PA_IDLE;
		          wait_count         <= 0;
		        end

	      // PA_REQ state : When using dynamic alignment PhaseAlignRequest
	      // signal is triggered in this state
		PA_REQ:
		  begin
		        PhaseAlignRequest  <= 1;
		        state_snk          <= PA_CMPLT;
		        wait_count         <= 0;
		  end          	

		// PA_CMPLT state : Waits for the PhaseAlignComplete signal.
		// If the signal is not received within 30 cycle, it returns to
		// the PA_REQ state.
		PA_CMPLT:
	       if (wait_count == 100)
		    begin
			PhaseAlignRequest  <= 0;
			wait_count         <= 0;
		       if (PhaseAlignComplete == 1)
				state_snk    <= PA_IDLE;
	            else
        			state_snk    <= PA_REQ;
		    end
		  else
		    begin
			PhaseAlignRequest  <= 0;
		      state_snk          <= PA_CMPLT;
		      wait_count         <= wait_count  + 1;
	         end
        
	    endcase
end

assign	PL4Rst_n = PL4Rst_n_int;
assign	CalProg = PL4Rst_n_int;
assign	SrcEn = SrcEn_i;
assign	SnkEn = SnkEn_i;



endmodule
