-------------------------------------------------------------------------------
--
--   File Name:  spi3_to_spi4_arbiter.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi3_to_spi4_arbiter.vhd file is to take 
--               the burst data from the four SPI-3 burst storage blocks and 
--               arbitrate writes into the SPI-4.2 Source Interface.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;
use work.spi_pkg.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi3_to_spi4_arbiter is
  port (
    -- System signals
    Reset_n                : in  std_logic;
    BridgeClk              : in  std_logic;
    SPI4_Status            : in  std_logic_vector(7 downto 0);
    SPI4_AlmostFull_n      : in  std_logic;
    
    -- Burst Storage Outputs
    BrstRdEn               : out  std_logic_vector(3 downto 0);
    BrstRdChan             : out  std_logic_vector(1 downto 0);
    SPI4WrEn               : out std_logic;
    
    -- Burst Storage Inputs    
    BrstRdEOP              : in   std_logic_vector(3 downto 0);
    BrstValid              : in   std_logic_vector(3 downto 0);
    BrstEmpty              : in   std_logic_vector(3 downto 0);
    BrstAlmostEmpty        : in   std_logic_vector(3 downto 0);
    BrstPktCntEq0          : in   std_logic_vector(3 downto 0);
    BrstPktCntEq1          : in   std_logic_vector(3 downto 0)
    );
end spi3_to_spi4_arbiter;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                   : System Level Reset (active low)
--    BridgeClk                 : Bridge Clock 
--
-- SPI4 Status Input Signal
--    SPI4_Status[7:0]          : SPI4 Source Status signal
--
-- SPI4 Source FIFO Almost Full Signal
--    SPI4_AlmostFull_n         : SPI4 Source FIFO Almost Full signal (active
--                                low)
--
-- Burst Storage Outputs
--    BrstRdEn[3:0]             : Burst Storage read enable signal
--    BrstRdChan[1:0]           : Burst Storage read channel
--     
-- Burst Storage Input Signals
--    BrstRdEOP                 : Burst Storage Read EOP - indicates if next
--                                read Burst Storage word has EOP
--    BrstRdValid               : Burst Storage Read Valid - indicates if Burst
--                                Storage has valid data
--    BrstAlmostEmpty           : Burst Storage is Almost Empty
--    BrstPktCntEq0[3:0]        : Burst Storage has zero EOPs (no complete
--                                packet)
--    BrstPktCntEq1[3:0]        : Burst Storage has only one EOP
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi3_to_spi4_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi3_to_spi4_arbiter is

  ------------------------------------------------------------------------------
  -- Type Declarations
  ------------------------------------------------------------------------------
  
  type statustype is array (0 to 3) of std_logic_vector(1 downto 0);
  type fsm_state is (SET_UP, TRANSFER, END_TRANSFER);
  
  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------
  --SPI4 Status signals
  signal SPI4ChStatus       : statustype;
  signal Status             : std_logic_vector(1 downto 0);

  --Arbitration signals
  signal Chan_int           : integer range 0 to 3;
  signal NxChan_int         : integer range 0 to 3;
  signal SchCh              : std_logic_vector(1 downto 0);  
  signal NextChan           : std_logic_vector(1 downto 0);
  signal NextChValid        : std_logic;
  signal NextChValidEarly   : std_logic;
  signal NextChDecode       : std_logic_vector(3 downto 0);
  signal NextChTaken        : std_logic;
  signal RdEn               : std_logic_vector(3 downto 0);
  signal CurrChan           : std_logic_vector(1 downto 0);
  signal State              : fsm_state;
  
  --Tx Burst Count signals
  signal TxBurstLimit       : std_logic_vector(8 downto 0);
  signal TxBurstCnt         : std_logic_vector(8 downto 0);
  signal TxBurstCntEn       : std_logic;
  signal TxBurstEnd         : std_logic;
  
  --SPI4 Source Write strobe signals
  signal WrEn               : std_logic;
  
begin

  ------------------------------------------------------------------------------
  -- Process to capture SPI4 Status for all 4 channels
  ------------------------------------------------------------------------------
  pl4status : process ( Reset_n, BridgeClk )
  begin
    if ( Reset_n = '0' ) then
      SPI4ChStatus  <= (others => "10") ;
    elsif ( BridgeClk = '1' and BridgeClk'event ) then
      for i in 0 to 3 loop
        if(SPI4_AlmostFull_n = '0') then
          SPI4ChStatus(i) <= "10" ;
        else
          SPI4ChStatus(i) <= SPI4_Status(2*i+1 downto 2*i) ;
        end if;
      end loop;  
    end if;  
  end process ;

  
  ------------------------------------------------------------------------------
  -- Process to find next eligible channel to transfer data from Burst Storage
  -- to SPI4 Source FIFO
  -- Channels are probed in round robin fashion
  -- Channel is eligible for transfer if that channel's
  --     - Burst Storage is not Almost Empty OR
  --     - Has at least one EOP in Burst Storage (BrstPktCntEq0 = '0')
  --     - Burst Storage is not Empty
  --     - SPI4 Channel Status received is not SATISFIED ("10")
  ------------------------------------------------------------------------------
  schaddr : process ( Reset_n, BridgeClk )
  begin
    if ( Reset_n = '0' ) then
      SchCh  <= "00" ;
      NextChan <= "00" ;
      NextChValid <= '0' ;
      NextChDecode <= "0000" ;
    elsif ( BridgeClk = '1' and BridgeClk'event ) then
        if(NextChValid = '0' or NextChTaken = '1') then
          SchCh <= SchCh + '1' ;
          NextChan <= conv_std_logic_vector(Chan_int, 2);
          NextChValid <= (((not BrstAlmostEmpty(Chan_int)) or
                          (not BrstPktCntEq0(Chan_int))) and
                          (not BrstEmpty(Chan_int)) and
                          (not SPI4ChStatus(Chan_int)(1))) ; 

          for i in 0 to 3 loop
            if(i = Chan_int) then
              NextChDecode(i) <= '1' ;
            else
              NextChDecode(i) <= '0' ;
            end if;
          end loop;
        else
          NextChValid <= (((not BrstAlmostEmpty(NxChan_int)) or
                          (not BrstPktCntEq0(NxChan_int))) and
                          (not BrstEmpty(NxChan_int)) and
                          (not SPI4ChStatus(NxChan_int)(1))) ;     
        end if;
    end if;  
  end process ;

 
  Chan_int <= conv_integer(SchCh);
  NxChan_int <= conv_integer(NextChan);

  
  NextChTaken <= '1' when ((NextChValid = '1') and State = SET_UP) or
                          (TxBurstCntEn = '0' and State = TRANSFER) else '0';

  TxBurstCntEn <= '1' when ((State = TRANSFER and TxBurstEnd = '0') and
                         BrstRdEOP(conv_integer(CurrChan)) = '0') else '0';

  NextChValidEarly <= (((not BrstPktCntEq1(NxChan_int)) and
                        (not BrstPktCntEq0(NxChan_int))) or
                      (BrstPktCntEq1(NxChan_int) and
                      (not BrstRdEOP(NxChan_int))) or 
                      (BrstPktCntEq0(NxChan_int) and 
                      (not BrstAlmostEmpty(NxChan_int))));    


  ------------------------------------------------------------------------------
  -- FSM to generate read strobes for Burst Storage FIFOs.
  -- SET_UP state      : This state is idle state and no read enables are 
  --                     generated until one of the channel is eligible for
  --                     transfer
  -- TRANSFER state    : This is the TRANSFER state where read enable are
  --                     generated for eligible channel's Burst Storage FIFOs.
  --                     Transfer for a channel is ended when "MaxBurst1" or
  --                     "MaxBurst2" credits of data or an EOP is transfered
  -- END_TRANSFER state: This ends reading from Burst Storage FIFOs.
  ------------------------------------------------------------------------------
  txfsm : process ( Reset_n, BridgeClk)
  begin
    if (Reset_n = '0') then
       State <= SET_UP ;
       RdEn  <= "0000" ;
       CurrChan <= "00" ;
    elsif(BridgeClk = '1' and BridgeClk'event) then
      case State is
        -----------------------------------------------------------------------
        -- State 0: SET_IP (Idle) state 
        -----------------------------------------------------------------------
        when SET_UP =>
          if(NextChValid = '1') then
            State <= TRANSFER ;
            RdEn <= NextChDecode ;
            CurrChan <= NextChan ;
           else
             State <= SET_UP ;
             RdEn <= "0000" ;
             CurrChan <= NextChan ;
           end if;

        -----------------------------------------------------------------------
        -- State 1: TRANSFER State 
        -----------------------------------------------------------------------        
        when TRANSFER =>
            if(TxBurstEnd = '1' or
               BrstRdEOP(conv_integer(CurrChan)) = '1') then
              if(NextChValid = '1' and NextChValidEarly = '1') then
                 State <= TRANSFER ;
                 RdEn <= NextChDecode ;
                 CurrChan <= NextChan ;
               else
                 State <= END_TRANSFER ;
                 RdEn <= "0000" ;
                 CurrChan <= NextChan ;
               end if;
             else
               State <= TRANSFER ;
             end if;

        -----------------------------------------------------------------------
        -- State 2: END_TRANSFER State 
        -----------------------------------------------------------------------
        when END_TRANSFER =>
                 State <= SET_UP ;
                 RdEn <= "0000" ;
                 CurrChan <= NextChan ;

        -----------------------------------------------------------------------
        -- NOP state 
        -----------------------------------------------------------------------
        when others => null;
      end case;
    end if;
  end process;


  ------------------------------------------------------------------------------
  -- Process to capture the status from SPI4 Source for Current Channel
  -- SPI4 Source Status is Starving ("00") or Hungry("01")
  ------------------------------------------------------------------------------
 statusreg: process(Reset_n, BridgeClk)
   begin
     if(Reset_n = '0') then
       Status <= "10" ;
     elsif (BridgeClk'event and BridgeClk = '1') then
       Status <= SPI4ChStatus(conv_integer(NextChan)) ;
     end if;
   end process;

  ------------------------------------------------------------------------------
  -- Process to count number of SPI4 Words (8 bytes) that have
  -- been transferred from channel's Burst Storage to SPI4 Source FIFOs
  -- MaxBurst1 and MaxBurst2 are indicated in number or credits (16 Bytes)
  -- MaxBurst1 should be > 1 (16 bytes)
  ------------------------------------------------------------------------------
  maxburstcnt: process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        TxBurstLimit <= (others => '0') ;
        TxBurstCnt <= (others => '0') ;
        TxBurstEnd <= '0' ;
      elsif(BridgeClk = '1' and BridgeClk'event) then
        if(TxBurstCntEn = '0') then
          if(Status = "00") then
            TxBurstLimit <= conv_std_logic_vector(2*MaxBurst1, 9) ;
          else
            TxBurstLimit <= conv_std_logic_vector(2*MaxBurst2, 9) ;
          end if;
        end if;

        if(TxBurstCntEn = '0') then
          TxBurstCnt <= (others => '0') ;
        else
          TxBurstCnt <= TxBurstCnt + 1 ;
        end if;

        if(TxBurstCntEn = '0') then
          TxBurstEnd <= '0' ;
        elsif(TxBurstCnt = TxBurstLimit-2) then
          TxBurstEnd <= '1' ;
        end if;
      end if;
    end process;
    
  ------------------------------------------------------------------------------
  -- Process to generate Write strobe for SPI4 Source FIFO
  ------------------------------------------------------------------------------
 spi4wr_strobe: process(Reset_n, BridgeClk)
   begin
     if(Reset_n = '0') then
       WrEn <= '0' ;
     elsif (BridgeClk'event and BridgeClk = '1') then
       WrEn <= RdEn(conv_integer(CurrChan)) ;
     end if;
   end process;
    
  ------------------------------------------------------------------------------
  -- Output Signal Assignments
  ------------------------------------------------------------------------------

    BrstRdEn <= RdEn;
    BrstRdChan <= CurrChan;
    SPI4WrEn   <= WrEn;
   
end xilinx;
