-------------------------------------------------------------------------------
-- SPI3 to SPI4 Burst Storage File
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--
--   File Name:  spi3_to_spi4_burst_storage.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi3_to_spi4_burst_storgae.vhd file is to
--               store a burst of data from the SPI3 interface prior to making 
--               it available to the SPI4.2 interface. 
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.spi_pkg.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi3_to_spi4_burst_storage is
  port (
    -- System signals
    Reset_n               : in  std_logic;
    BridgeClk             : in  std_logic;
  
    -- SPI3 Burst Storage Interface Output Signals
    SPI3_Addr             : in  std_logic_vector(7 downto 0);
    SPI3_Data             : in  std_logic_vector(31 downto 0);
    SPI3_Mod              : in  std_logic_vector(1 downto 0);
    SPI3_SOP              : in  std_logic;
    SPI3_EOP              : in  std_logic;
    SPI3_Err              : in  std_logic;
    SPI3_Valid            : in  std_logic;
	 SPI3_Src_Rdy			  : in  std_logic;
    
    -- Burst Storage Interface Output Signals
    BrstFull_n            : out std_logic;

    -- Burst Storage Inteface Output Signals
    BrstAddr              : out std_logic_vector(7 downto 0);
    BrstData              : out std_logic_vector(63 downto 0);
    BrstMod               : out std_logic_vector(2 downto 0);
    BrstSOP               : out std_logic;
    BrstEOP               : out std_logic;
    BrstErr               : out std_logic;
    
    -- Burst Storage Read Enable
    BrstRdEn              : in  std_logic;
    
    BrstAlmostEmpty       : out std_logic;
    BrstEmpty             : out std_logic;
    BrstValid             : out std_logic;
    BrstRdEOP             : out std_logic;
    
    --Packet Count Signals
    PktCntEq0             : out std_logic;
    PktCntEq1             : out std_logic
    );
end spi3_to_spi4_burst_storage;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n               : System Level Reset (active low)
--    BridgeClk             : BridgeClk Clock 
--
-- Burst Storage Input Signals from SPI3 Sink FIFOs
--    SPI3_Addr             : SPI3 Sink Address 
--    SPI3_Data             : SPI3 Sink Data 
--    SPI3_Mod              : SPI3 Sink MOD 
--    SPI3_SOP              : SPI3 Sink Start of Packet 
--    SPI3_EOP              : SPI3 Sink End of Packet  
--    SPI3_Err              : SPI3 Sink Error  
--    SPI3_Valid            : SPI3 Sink Data is Valid  
--
-- Burst Storage Output Signal - SPI3 Read Interface 
--    BrstFull_n            : Burst Storage FIFO Full (active low) 
--
-- Burst Storage Output Signals - SPI4 Write Interface 
--    BrstAddr              : Burst Storage Address 
--    BrstData              : Burst Storage Data 
--    BrstMod               : Burst Storage MOD 
--    BrstSOP               : Burst Storage Start of Packet 
--    BrstEOP               : Burst Storage End of Packet  
--    BrstErr               : Burst Storage Error  
--
-- Burst Storage Input Signals - Arbiter Interface
--    BrstRdEn              : Burst Storage Read Enable
--
-- Burst Storage Output Signals - Arbiter Interface   
--    BrstAlmostEmpty       : Burst Storage is Almost Empty
--    BrstEmpty             : Burst Storage Empty
--    BrstRdValid           : Burst Storage Read Valid - indicates if Burst Storage
--                            has valid data
--    BrstRdEOP             : Burst Storage Read EOP - indicates if next Burst
--                            Storage word has an EOP
--    BrstPktCntEq0[3:0]    : Burst Storage has zero EOPs (no complete packet)
--    BrstPktCntEq1[3:0]    : Burst Storage has one EOP
--
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi3_to_spi4_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi3_to_spi4_burst_storage is

  ------------------------------------------------------------------------------
  -- Component Declarations
  ------------------------------------------------------------------------------
  component generic_sfifo_512x72
    port (
	   clk       : IN std_logic;
	   sinit     : IN std_logic;
	   din       : IN std_logic_VECTOR(71 downto 0);
	   wr_en     : IN std_logic;
	   rd_en     : IN std_logic;
	   dout      : OUT std_logic_VECTOR(71 downto 0);
	   full      : OUT std_logic;
	   empty     : OUT std_logic;
	   data_count: OUT std_logic_VECTOR(8 downto 0));
  end component;

  --Synplify Synthesis attributes
  attribute syn_black_box      : boolean ;
  attribute syn_black_box of generic_sfifo_512x72  : component is true;
  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------
  --SPI3 Interface Register signals
  signal SPI3_Data_r0    : std_logic_vector(31 downto 0);
  signal SPI3_Mod_r0     : std_logic_vector(1 downto 0);
  signal SPI3_Valid_r0   : std_logic;
  signal SPI3_Valid_r1   : std_logic;
  signal SPI3_SOP_r0     : std_logic;
  signal SPI3_SOP_r1     : std_logic;
  signal SPI3_EOP_r0     : std_logic;
  signal SPI3_EOP_r1     : std_logic;
  signal SPI3_Err_r0     : std_logic;
  signal SPI3_Err_r1     : std_logic;
  signal SPI3_Src_Rdy_r0 : std_logic;

  --Signal to convert two SP-3 words into one 64 bit word
  signal ConvEn          : std_logic;
  signal ConvEn_r0 		 : std_logic;
  signal SPI3_Data_d1    : std_logic_vector(31 downto 0);
  signal SPI3_Data_d2    : std_logic_vector(31 downto 0);
  signal Mod_3bits       : std_logic_vector(2 downto 0);

  --Bridge FIFO  signals
  signal FIFO_Reset      : std_logic;
  signal FIFOWr          : std_logic;
  signal FIFOWr_r0       : std_logic;
  signal FIFOWr_r1       : std_logic;  
  signal FIFO_Write      : std_logic;
  signal FIFO_Din        : std_logic_vector(71 downto 0);
  signal FIFO_RdEn       : std_logic;
  signal FIFO_Dout       : std_logic_vector(71 downto 0);
  signal FIFO_Full       : std_logic;
  signal FIFO_Empty      : std_logic;
  signal FIFO_AlmostEmpty: std_logic;
  signal Empty           : std_logic;
  signal Full            : std_logic;  
  signal DataCount       : std_logic_vector(8 downto 0); 

  --RegBank signals
  signal preload_stage1  : std_logic;
  signal preload_stage2  : std_logic;
  signal data_valid      : std_logic;
  signal data_valid_r0   : std_logic;
  signal RdEn            : std_logic;
  signal RdEn_reg        : std_logic;
  signal RegBank         : std_logic_vector(71 downto 0);
  signal Taken           : std_logic;

  --Pkt Count signals
  signal Wr_EOP_r0       : std_logic;
  signal Wr_EOP_r1       : std_logic;
  signal Rd_EOP          : std_logic;
  signal PktCntEn        : std_logic;
  signal PktAdd_Val      : std_logic_vector(8 downto 0);
  signal PktCnt          : std_logic_vector(8 downto 0);
  signal PktCnt_Eq0      : std_logic;
  signal PktCnt_Eq1      : std_logic;


  
begin

 -------------------------------------------------------------------------------
 --Register SPI3 Sink FIFO outputs 
 -------------------------------------------------------------------------------
  spi3fifo_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        SPI3_Data_r0 <= (others => '0') after TFF;
        SPI3_Mod_r0  <= "00" after TFF;
        SPI3_Valid_r0 <= '0' after TFF;
		  SPI3_Valid_r1 <= '0' after TFF;
		  SPI3_SOP_r0 <= '0' after TFF;
        SPI3_EOP_r0 <= '0' after TFF;
        SPI3_Err_r0 <= '0' after TFF;
		  SPI3_Src_Rdy_r0 <= '0' after TFF;
		  ConvEn_r0    <= '0' after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
        SPI3_Data_r0 <= SPI3_Data after TFF;
        SPI3_Mod_r0  <= SPI3_Mod  after TFF;
        SPI3_Valid_r0 <= SPI3_Valid after TFF;
		  SPI3_Valid_r1 <= SPI3_Valid_r0 after TFF;
        SPI3_SOP_r0 <= SPI3_SOP and SPI3_Src_Rdy and SPI3_Valid after TFF;
        SPI3_EOP_r0 <= SPI3_EOP and SPI3_Src_Rdy and SPI3_Valid after TFF;
        SPI3_Err_r0 <= SPI3_Err and SPI3_Src_Rdy and SPI3_Valid after TFF;  
		  SPI3_Src_Rdy_r0 <= SPI3_Src_Rdy after TFF;
		  ConvEn_r0    <= ConvEn after TFF;
      end if;
    end process;

  -----------------------------------------------------------------------------
  -- Generate enable signal to convert two SPI-3 FIFO words (32 bits) to one
  -- SPI-4 FIFO word (64 bits)
  -----------------------------------------------------------------------------
--  conv_en : process(Reset_n, BridgeClk)
--    begin
--      if(Reset_n = '0') then
--        ConvEn <= '1' after TFF;
--      elsif ( BridgeClk = '1' and BridgeClk'event ) then
--        if(SPI3_Src_Rdy = '1') then
--          if(SPI3_SOP = '1' ) then
--            ConvEn <= '0' after TFF;
--          elsif (SPI3_valid_r0 = '1' and SPI3_valid = '1') then
--            ConvEn <= not ConvEn after TFF;
--          end if;
--        end if;
--      end if;
--    end process;

  conv_en : process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        ConvEn <= '1' after TFF;
      elsif (BridgeClk = '1' and BridgeClk'event ) then
        if(SPI3_Src_Rdy = '1' and SPI3_SOP = '1' and SPI3_valid = '1') then
          --if(SPI3_SOP = '1' ) then
            ConvEn <= '0' after TFF;
        elsif (SPI3_Src_Rdy = '1' and SPI3_valid = '1') then
            ConvEn <= not ConvEn after TFF;
          --end if;
        end if;
      end if;
    end process;

  -----------------------------------------------------------------------------
  -- Convert two 32-bit SPI-3 Data words to one 64-bit word.
  -- If odd SPI3 Word has EOP, assign lower SPI4 word to zeros. Otherwise
  -- odd SPI3 words are assigned as 32 MSB bits of SPI4 word  [63:32]and
  -- even SPI3 words are assigned to lower LSB bits of SPI4 word [31:0]. 
  -----------------------------------------------------------------------------
  spi3data_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        SPI3_Data_d1 <= (others => '0') after TFF;
        SPI3_Data_d2 <= (others => '0') after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
        if(SPI3_Src_rdy_r0 = '1' and SPI3_Valid_r0 = '1') then
          if(ConvEn = '0' and SPI3_EOP_r0 = '1') then
           SPI3_Data_d1 <= SPI3_Data_r0 after TFF;
           SPI3_Data_d2 <= (others => '0') after TFF;
          elsif(ConvEn = '0' and (SPI3_EOP_r0 = '0' or SPI3_SOP_r0 = '1')) then
            SPI3_Data_d1 <= SPI3_Data_r0 after TFF;
          elsif ( SPI3_Valid_r0 = '1' ) then
            SPI3_Data_d2 <= SPI3_Data_r0 after TFF;
          end if;
        end if;
      end if;
    end process;


  ------------------------------------------------------------------------------
  -- SPI3 Mod conversion to SPI-4 Mod 
  -- This process converts Mod for two SPI-3 FIFO words to Mod of one SPI4 
  -- FIFO word. 2-bit Mod of SPI-3 for SPI-3 data [31:0] is converted to 3-bit
  -- SPI4 Mod.
  ------------------------------------------------------------------------------
  spi3mod_conv : process ( Reset_n, BridgeClk )
  begin
    if ( Reset_n = '0' ) then
     Mod_3bits <= "000" after TFF;    
    elsif ( BridgeClk = '1' and BridgeClk'event ) then
      if(SPI3_Src_Rdy_r0 = '1' and SPI3_EOP_r0 = '1') then
        if(ConvEn = '0') then
          case SPI3_Mod_r0 is
             when "11" => Mod_3bits <= "100" after TFF;
             when "10" => Mod_3bits <= "011" after TFF;
             when "01" => Mod_3bits <= "010" after TFF;
             when "00" => Mod_3bits <= "001" after TFF;
             when others => null;
           end case;
         else
          case SPI3_Mod_r0 is
             when "11" => Mod_3bits <= "000" after TFF;
             when "10" => Mod_3bits <= "111" after TFF;
             when "01" => Mod_3bits <= "110" after TFF;
             when "00" => Mod_3bits <= "101" after TFF;
             when others => null;
           end case;           
        end if;
      else
         Mod_3bits <= "000" after TFF;
      end if;
    end if;
  end process;

  ------------------------------------------------------------------------------
  -- Process to capture SOP, EOP, Err bits from SPI3 Interface and
  -- write them to Burst Storage FIFO.
  ------------------------------------------------------------------------------
  
  fifopkt_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        SPI3_SOP_r1 <= '0' after TFF;
        SPI3_EOP_r1 <= '0' after TFF;
        SPI3_Err_r1 <= '0' after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
        if(SPI3_Src_Rdy_r0 = '1') then
          if(ConvEn = '0' and SPI3_Valid_r0 = '1') then
            SPI3_SOP_r1 <= SPI3_SOP_r0 after TFF;
				  else
						SPI3_SOP_r1 <= SPI3_SOP_r1;
          end if;

          if(SPI3_Err_r0 = '1' and SPI3_Valid_r0 = '1') then
             SPI3_Err_r1 <= '1' after TFF;
          elsif(SPI3_SOP_r0 = '1' and SPI3_Valid_r0 = '1') then
             SPI3_Err_r1 <= '0' after TFF;
          end if;

          SPI3_EOP_r1 <= SPI3_EOP_r0 after TFF;
        end if; 
      end if;
    end process;
 

  ------------------------------------------------------------------------------
  -- Process to generate Burst Storage  FIFO write strobe
  ------------------------------------------------------------------------------
  spi3fifowr_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
       FIFOWr <= '0' after TFF;
		 FIFOWr_r0 <= '0' after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
        if(SPI3_valid_r0 = '1' and SPI3_Src_Rdy_r0 = '1') then
          if(ConvEn = '0' and SPI3_EOP_r0 = '1') then
            FIFOWr <= '1' after TFF;
          elsif(ConvEn = '0') then
            FIFOWr <= '0' after TFF;
          else
            FIFOWr <= '1' after TFF;
          end if;
        else
          FIFOWr <= '0' after TFF;    
        end if;
        FIFOWr_r0 <= FIFOWr after TFF;
        FIFOWr_r1 <= FIFOWr_r0 after TFF;
      end if;
    end process;
	 
  
  -----------------------------------------------------------------------------
  -- Combinatorial Burst FIFO  Signal Assignments
  -----------------------------------------------------------------------------

  --FIFO Data input Bus Bit Mappings

  --FIFO_Din[71:0]  --> Burst Storage FIFO Input
  --   -Packet Data    -> FIFO_Din(63 downto 0)
  --   -Mod            -> FIFO_Din(66 downto 64)
  --   -Err            -> FIFO_Din(67)
  --   -SOP            -> FIFO_Din(68)
  --   -EOP            -> FIFO_Din(69)
  --   -EOP duplicate1 -> FIFO_Din(70) -- To improve timing
  --   -EOP duplicate2 -> FIFO_Din(71) -- To improve timing
  fifo_data :  process(Reset_n, BridgeClk)
	begin
      if(Reset_n = '0') then
			FIFO_Din(71 downto 0)<= x"000000000000000000";
			FIFO_Write  <= '1';
		else
			FIFO_Din(71 downto 0) <= SPI3_EOP_r1 & SPI3_EOP_r1 & SPI3_EOP_r1 &
                             SPI3_SOP_r1 & SPI3_Err_r1 & Mod_3bits &
                             SPI3_Data_d1 & SPI3_Data_d2 ;
--		FIFO_Write <= (FIFOWr) and (not FIFOWr_r0);
		FIFO_Write <= (FIFOWr);
	 
	 end if;
	 end process;

    FIFO_Reset <= not Reset_n;
 ------------------------------------------------------------------------------
 -- Burst Storage FIFO
 ------------------------------------------------------------------------------

burst_fifo : generic_sfifo_512x72
		port map (
			clk        => BridgeClk,
			sinit      => FIFO_Reset,
			din        => FIFO_Din,
			wr_en      => FIFO_Write,
			rd_en      => FIFO_RdEn,
			dout       => FIFO_Dout,
			full       => FIFO_Full,
			empty      => FIFO_Empty,
			data_count => DataCount);

  -----------------------------------------------------------------------------
  -- Process to generate Burst Storage  FIFO Almost Empty signal
  -----------------------------------------------------------------------------
  spi3fifoae_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
       FIFO_AlmostEmpty <= '1' after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
        if(DataCount > SPI3_SPI4BrstAE_Thresh) then
          FIFO_AlmostEmpty <= '0' after TFF;
        else
          FIFO_AlmostEmpty <= '1' after TFF;
        end if;
      end if;
    end process;

  -----------------------------------------------------------------------------
  -- Process to generate Burst Storage  FIFO Empty signal
  -----------------------------------------------------------------------------
      
  spi3fifoempty_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        Empty <= '1' after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
         if((FIFO_Empty = '1'and Taken = '1' and data_valid_r0 = '1') or
            data_valid_r0 = '0') then
          Empty <= '1' after TFF;
         else
          Empty <= '0' after TFF;
        end if;
      end if;
    end process;

  -----------------------------------------------------------------------------
  -- Process to generate Burst Storage  FIFO Full signal
  -- Assert FIFO Full if DataCount > 508
  -----------------------------------------------------------------------------
  
  spi3fifofull_reg:  process(Reset_n, BridgeClk)
    begin
      if(Reset_n = '0') then
        Full <= '0' after TFF;
      elsif ( BridgeClk = '1' and BridgeClk'event ) then
        if(DataCount(8 downto 3) = "111111") then
          Full <= '1' after TFF;
        else
          Full <= '0' after TFF;
        end if;
      end if;
    end process;
    
  -----------------------------------------------------------------------------
  -- FIFO Read Logic
  -- The following logic implement a "Taken FIFO". There is a register bank
  -- REGBANK in the FIFO output and this REGBANK is pre-loaded with FIFO Output 
  -- Whenever FIFO is not empty
  -- the first word is pre-read and loaded into a register bank REGBANK. 
  -----------------------------------------------------------------------------
  Taken <= BrstRdEn;            
  preload_stage1 <= (not FIFO_Empty) and ((not data_valid) or
                    (not data_valid_r0));
  preload_stage2 <= ((not data_valid_r0) and data_valid);

  --Only do a read from the FIFO when the FIFO is not currently empty
  --and FIFO wasn't empty on previous cycle.
  RdEn <= (Taken and (not FIFO_Empty)) or preload_stage1;
  RdEn_reg <= (Taken or preload_stage2);

  FIFO_RdEn <= RdEn;

    
  -----------------------------------------------------------------------------
  -- Generate valid strobe for Stage 1
  -----------------------------------------------------------------------------

  Stage1 : process (Reset_n, BridgeClk)
  begin
    if Reset_n = '0' then
      data_valid <= '0' after TFF;
    elsif BridgeClk'event and BridgeClk = '1' then
      if RdEn = '1' then      --reading data out of RAM into RAM output register
        data_valid <= '1' after TFF;
      elsif RdEn_reg = '1' then    --reading data out of register and not out of RAM register
        data_valid <= '0' after TFF;
      end if;
    end if;
  end process Stage1;

  ------------------------------------------------------------------------------
  -- Generate data valid strobe to indicate when REGBANK has valid data.
  -- Register data from the FIFO into REGBANK
  ------------------------------------------------------------------------------

  Stage2 : process (Reset_n, BridgeClk)
  begin
    if Reset_n = '0' then
      RegBank     <= (others => '0') after TFF;
      data_valid_r0 <= '0'             after TFF;
    elsif BridgeClk'event and BridgeClk = '1' then
      if RdEn_reg = '1' then  --reading data out of register and 
                              --possibly into register from FIFO
        --Zero out data if register won't contain valid data
        if (data_valid = '1') then
          RegBank <= FIFO_Dout after TFF;
        else
          RegBank <= (others => '0') after TFF;
        end if;
        data_valid_r0 <= data_valid after TFF;
      end if;
    end if;
  end process Stage2;

    --Burst Storage Outputs
    BrstEmpty <=  Empty;
    BrstAlmostEmpty <=  FIFO_AlmostEmpty;
    BrstFull_n <= not Full;
    
    BrstValid <= data_valid_r0;


  --RegBank[71:0]  --> Burst Storage FIFO Output register
  --   -Packet Data    -> RegBank(63 downto 0)
  --   -Mod            -> RegBank(66 downto 64)
  --   -Err            -> RegBank(67)
  --   -SOP            -> RegBank(68)
  --   -EOP            -> RegBank(69)
  --   -EOP duplicate1 -> RegBank(70) -- To improve timing
  --   -EOP duplicate2 -> RegBank(71) -- To improve timing
  
    BrstData  <= RegBank(63 downto 0);       
    BrstMod   <= RegBank(66 downto 64);          
    BrstErr   <= RegBank(67);          
    BrstSOP   <= RegBank(68);         
    BrstEOP   <= RegBank(69);
    BrstRdEOP  <= RegBank(70);
-------------------------------------------------------------------------------
-- Generate EOP count in the Burst Storage FIFO.
-- Process to generate write EOP strobe
-------------------------------------------------------------------------------

  create_wrpkt_strobe : process (BridgeClk, Reset_n)
  begin
    if (Reset_n = '0') then
       Wr_EOP_r0  <= '0' after TFF;
       Wr_EOP_r1 <= '0' after TFF;
    elsif (BridgeClk'event and BridgeClk = '1') then
       Wr_EOP_r0  <= FIFO_Write and FIFO_Din(71)   after TFF;
       Wr_EOP_r1  <= Wr_EOP_r0 and (not FIFO_Full) after TFF;
    end if;
  end process create_wrpkt_strobe;


  ------------------------------------------------------------------------------
  --Count number of EOPs in the Burst Storage FIFO
  --EOP count : Increment EOP count if no EOP is read out while writing another
  --            EOP into the FIFO
  --            Decrement EOP count if EOP is read out while no other EOP is
  --            being written into the FIFO
  ------------------------------------------------------------------------------
  Rd_EOP   <= Taken and RegBank(71);  --EOP will not be '1' if read not valid
  PktCntEn <= (Taken and RegBank(71)) xor Wr_EOP_r1;

  --add -1 if read & not write,add 1 if not read & write
  PktAdd_Val(0) <= '1';
  addval_gen: for  i in 1 to 8 generate
    PktAdd_Val(i) <= not Wr_EOP_r1;
  end generate;

  create_pkt_count : process (BridgeClk, Reset_n)
  begin
    if (Reset_n = '0') then
      PktCnt <= (others => '0') after TFF;
    elsif (BridgeClk'event and BridgeClk = '1') then
      if PktCntEn = '1' then
        PktCnt <= PktCnt + PktAdd_Val after TFF;
      end if;
    end if;
  end process create_pkt_count;

-------------------------------------------------------------------------------
-- Generate Packet Count Equal to Zero and Packet count Equal to one signals
-- These signals indicate whether one or more EOPs are present in the Burst FIFO
-- PktCntEq0 = '0'  indicates No EOPs are present,
--           = '1'  indicates atleast  one EOP is present
-- PktCntEq1 = '1'  indicates only one EOP is present in the Burst FIFO,
--           = '0'  indicates either zero EOPs or more than one EOPs are present in the Burst FIFO
-------------------------------------------------------------------------------

  create_pkt_cnt : process (BridgeClk, Reset_n)
  begin
    if (Reset_n = '0') then
      PktCnt_Eq1 <= '0' after TFF;
      PktCnt_Eq0 <= '1' after TFF;
    elsif (BridgeClk'event and BridgeClk = '1') then
      if Rd_EOP = '1' and Wr_EOP_r1 = '0' then
        if PktCnt = 2 then
          PktCnt_Eq1 <= '1' after TFF;
        else
          PktCnt_Eq1 <= '0' after TFF;
        end if;
        PktCnt_Eq0 <= PktCnt_Eq1 after TFF;
      elsif Rd_EOP = '0' and Wr_EOP_r1 = '1' then
        PktCnt_Eq1 <= PktCnt_Eq0 after TFF;
        PktCnt_Eq0 <= '0'   after TFF;
      end if;
    end if;
  end process create_pkt_cnt;

  PktCntEq0 <= PktCnt_Eq0;
  PktCntEq1 <= PktCnt_Eq1;
  
end xilinx;