-------------------------------------------------------------------------------
--
--   File Name:  spi3_to_spi4_read.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi3_to_spi4_read.vhd file is to provide 
--               the SPI-3 core read logic. This logic will read data from one
--               SPI-3 interface as long as it is not empty, and the SPI3
--               burst storage is not full.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

library work;
use work.spi_pkg.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi3_to_spi4_read is
  port (
    -- System signals
    Reset_n               : in  std_logic;
    BridgeClk              : in  std_logic;

    -- SPI3 Core Interface Input Signals
    SPI3_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI3_FFData           : in  std_logic_vector(31 downto 0);
    SPI3_FFMod            : in  std_logic_vector(1 downto 0);
    SPI3_FFSOP            : in  std_logic;
    SPI3_FFEOP            : in  std_logic;
    SPI3_FFErr            : in  std_logic;

	 SPI3_Src_Rdy			  : in  std_logic;
	 
--    SPI3_FFEmpty_n        : in  std_logic;
--    SPI3_FFAlmostEmpty_n  : in  std_logic;

    -- SPI3 Core Interface Output Signals
--    SPI3_FFRdEn_n         : out std_logic;
--		Edit by Zhex. These signals are not used in v4.1
    SPI3_Dst_Rdy			  : out  std_logic;
    -- SPI3 Burst Storage Interface Input Signals
    SPI3_Full_n           : in  std_logic;

    -- SPI3 Burst Storage Interface Output Signals
    SPI3_Addr             : out std_logic_vector(7 downto 0);
    SPI3_Data             : out std_logic_vector(31 downto 0);
    SPI3_Mod              : out std_logic_vector(1 downto 0);
    SPI3_SOP              : out std_logic;
    SPI3_EOP              : out std_logic;
    SPI3_Err              : out std_logic;
    SPI3_Valid            : out std_logic
    );
end spi3_to_spi4_read;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n               : System Level Reset (active low)
--    BridgeClk              : SPI3 Interface Clock 

-- SPI3 Core Interface Input Signals
--    SPI3_FFAddr           : SPI3 Interface Address
--    SPI3_FFData           : SPI3 Interface Data
--    SPI3_FFMod            : SPI3 Interface MOD
--    SPI3_FFSOP            : SPI3 Interface Start of Packet 
--    SPI3_FFEOP            : SPI3 Interface End of Packet 
--    SPI3_FFErr            : SPI3 Interface Error 
--    SPI3_FFValid          : SPI3 Interface Data is Valid 
--    SPI3_FFEmpty_n        : SPI3 Interface is Empty (active low) 
--    SPI3_FFAlmostEmpty_n  : SPI3 Interface is Almost Empty (active low)

-- SPI3 Core Interface Output Signals
--    SPI3_FFRdEn_n         : SPI3 Interface Read Enable (active low)

-- SPI3 Burst Storage Interface Input Signals
--    SPI3_Full_n           : Burst Logic Full (active low) 

-- SPI3 Burst Storage Interface Output Signals
--    SPI3_Addr             : Burst Logic Address 
--    SPI3_Data             : Burst Logic Data 
--    SPI3_Mod              : Burst Logic MOD 
--    SPI3_SOP              : Burst Logic Start of Packet 
--    SPI3_EOP              : Burst Logic End of Packet  
--    SPI3_Err              : Burst Logic Error  
--    SPI3_Valid            : Burst Logic Data is Valid  
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi3_to_spi4_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi3_to_spi4_read is

  ------------------------------------------------------------------------------
  -- Constant Declarations
  ------------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  -- State Machine Variable and Type Declarations
  -- StateType: 
  --    IDLE             - Reset and idle state.
  --    REQUEST          - A data transfer request. This indicates that the
  --                       loopback is enabled, the Receive FIFO has data, and
  --                       the Transmit FIFO can accept data.
  --    TRANSFER         - Data is transferred in this state. Remain in this
  --                       state until the loopback is disabled,
  --                       the Receive FIFO
  --                       doesn't have any more data, or the
  --                       Transmit FIFO cannot
  --                       accept any more data.
  --    FIRST_TRANSFER     - Indicates that the Receive FIFO has the first word
  --                       available for reading.
  --
  -- Variables:
  --    State   - holds current state of state machine
  -----------------------------------------------------------------------------
  type StateType is (IDLE, REQUEST, TRANSFER, FIRST_TRANSFER);
  signal State : StateType;

  -----------------------------------------------------------------------------
  -- Signal Declarations
  -- OneWordLeft - Indicates that only one word is left in the FIFO, and a 
  --               single read should be performed
  -----------------------------------------------------------------------------
--  signal OneWordLeft            : std_logic;
--  signal SPI3_FFEmpty_n_r       : std_logic;
--  signal SPI3_FFAlmostEmpty_n_r : std_logic;
  signal SPI3_Dst_Rdy_int		  : std_logic;

begin

  -----------------------------------------------------------------------------
  -- Drive out the inputs from the Receive FIFO.
  -----------------------------------------------------------------------------
  SPI3_Data   <= SPI3_FFData;
  SPI3_Addr   <= SPI3_FFAddr;
  SPI3_Mod    <= SPI3_FFMod;
  SPI3_SOP    <= SPI3_FFSOP;
  SPI3_EOP    <= SPI3_FFEOP;
  SPI3_Err    <= SPI3_FFErr;
--  SPI3_Valid  <= SPI3_FFValid;
  SPI3_Dst_rdy <= SPI3_Dst_rdy_int;

  -----------------------------------------------------------------------------
  -- Determine if only one word is left in the FIFO. This is true only if 
  -- Empty_n was asserted on the previous cycle, and AlmostEmpty_n is not 
  -- asserted on the current cycle.
  -----------------------------------------------------------------------------
--  reg_empty: process (Reset_n, BridgeClk)
--  begin
--    if (Reset_n = '0') then
--      SPI3_FFEmpty_n_r       <= '0' ;
--      SPI3_FFAlmostEmpty_n_r <= '0' ;
--    elsif (BridgeClk'event and BridgeClk = '1') then
--      SPI3_FFEmpty_n_r       <= SPI3_FFEmpty_n ;
--      SPI3_FFAlmostEmpty_n_r <= SPI3_FFAlmostEmpty_n ;
--    end if;
--  end process reg_empty;

  -----------------------------------------------------------------------------
  -- In order for only one word left to be true, then not AE_n and E_n needs
  -- to be true for at least two cycles. This prevents flagging an incorrect
  -- one word left indication as the FIFO is transitioning from E to AE.
  -----------------------------------------------------------------------------

--    one_left: process (Reset_n, BridgeClk)
--     begin
--      if (Reset_n = '0') then
--        OneWordLeft <= '0' after TFF;
--      elsif (BridgeClk'event and BridgeClk = '1') then
--        OneWordLeft <= ((not SPI3_FFAlmostEmpty_n and SPI3_FFEmpty_n) and
--                       (not SPI3_FFAlmostEmpty_n_r and SPI3_FFEmpty_n_r));
--    end if;
--   end process one_left;


  -----------------------------------------------------------------------------
  -- State Machine Process
  -----------------------------------------------------------------------------
  state_machine : process ( Reset_n, BridgeClk )
  begin
    if(Reset_n = '0') then
      State         <= IDLE ;
      SPI3_Valid <= '0' ;
		SPI3_Dst_rdy_int <= '0';
    elsif (BridgeClk'event and BridgeClk = '1') then

      case State is
        -----------------------------------------------------------------------
        -- State 0: IDLE 
        -----------------------------------------------------------------------
        when IDLE =>
          if ((SPI3_Src_rdy = '1') and (SPI3_Full_n = '1')) then
            State         <= TRANSFER ;
            SPI3_Valid <= '1' ;
				SPI3_Dst_rdy_int <= '1';
				
          else
            State         <= IDLE ;
            SPI3_Valid <= '0' ;
				SPI3_Dst_rdy_int <= '0';
          end if;
        -----------------------------------------------------------------------
        -- State 1: REQUEST
        -----------------------------------------------------------------------
--        when REQUEST =>
--          State         <= TRANSFER ;
--          SPI3_FFRdEn_n <= '0' ;
        -----------------------------------------------------------------------
        -- State 2: TRANSFER
        -----------------------------------------------------------------------
        when TRANSFER =>
          if ((SPI3_SRC_rdy = '1') and (SPI3_Full_n = '1')) then
            State         <= TRANSFER ;
            SPI3_Valid <= '1' ;
				SPI3_Dst_rdy_int <= '1';
          else
            State         <= IDLE ;
            SPI3_Valid <= '0' ;
				SPI3_Dst_rdy_int <= '0';

          end if;
        -----------------------------------------------------------------------
        -- State 3: FIRST_TRANSFER
        -----------------------------------------------------------------------
--        when FIRST_TRANSFER =>
--          State         <= IDLE ;
--          SPI3_FFRdEn_n <= '1' ;
        -----------------------------------------------------------------------
        -- Error and Default States
        -----------------------------------------------------------------------
        when OTHERS =>
          State         <= IDLE ;
          SPI3_Valid <= '0' ;
			 SPI3_Dst_rdy_int <= '0';
      end case;
    end if;
  end process state_machine;

end xilinx;
