-------------------------------------------------------------------------------
--
--   File Name:  spi3_to_spi4_write.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi3_to_spi4_write.vhd file is to take the 
--               burst data from the four SPI-3 burst storage blocks and write 
--               it into the SPI-4.2 interface.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_misc.all;

use work.spi_pkg.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi3_to_spi4_write is
   
  port (
    -- System signals
    Reset_n               : in  std_logic;
    BridgeClk             : in  std_logic;

    -- SPI3 0 Write Logic Inteface Input Signals
    SPI3_0_BrstData       : in  std_logic_vector(63 downto 0);
    SPI3_0_BrstMod        : in  std_logic_vector(2 downto 0);
    SPI3_0_BrstSOP        : in  std_logic;
    SPI3_0_BrstEOP        : in  std_logic;
    SPI3_0_BrstErr        : in  std_logic;

    -- SPI3 1 Write Logic Inteface Input Signals
    SPI3_1_BrstData       : in  std_logic_vector(63 downto 0);
    SPI3_1_BrstMod        : in  std_logic_vector(2 downto 0);
    SPI3_1_BrstSOP        : in  std_logic;
    SPI3_1_BrstEOP        : in  std_logic;
    SPI3_1_BrstErr        : in  std_logic;

    -- SPI3 2 Write Logic Inteface Input Signals
    SPI3_2_BrstData       : in  std_logic_vector(63 downto 0);
    SPI3_2_BrstMod        : in  std_logic_vector(2 downto 0);
    SPI3_2_BrstSOP        : in  std_logic;
    SPI3_2_BrstEOP        : in  std_logic;
    SPI3_2_BrstErr        : in  std_logic;

    -- SPI3 3 Write Logic Inteface Input Signals
    SPI3_3_BrstData       : in  std_logic_vector(63 downto 0);
    SPI3_3_BrstMod        : in  std_logic_vector(2 downto 0);
    SPI3_3_BrstSOP        : in  std_logic;
    SPI3_3_BrstEOP        : in  std_logic;
    SPI3_3_BrstErr        : in  std_logic;

    -- SPI3 Arbiter Interface Input Signals
    ArbiterRdChan         : in  std_logic_vector(1 downto 0);
    ArbiterWrEn           : in  std_logic;

    -- SPI4.2 Interface Output Signals
    SPI4_WrAddr           : out std_logic_vector(7 downto 0);
    SPI4_WrData           : out std_logic_vector(63 downto 0);
    SPI4_WrMod            : out std_logic_vector(2 downto 0);
    SPI4_WrSOP            : out std_logic;
    SPI4_WrEOP            : out std_logic;
    SPI4_WrErr            : out std_logic;
    SPI4_WrEn_n           : out std_logic
    );
end spi3_to_spi4_write;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n               : System Level Reset (active low)
--    SPI3_BridgeClk              : SPI3 Interface Clock 
--
-- SPI3 Number 0 Write Logic Interface Input Signals
--    SPI3_0_BrstAddr       : Write Logic Address 
--    SPI3_0_BrstData       : Write Logic Data 
--    SPI3_0_BrstMod        : Write Logic MOD 
--    SPI3_0_BrstSOP        : Write Logic Start of Packet 
--    SPI3_0_BrstEOP        : Write Logic End of Packet  
--    SPI3_0_BrstErr        : Write Logic Error  
--    SPI3_0_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
--                            (active low)
--
-- SPI3 Number 1 Write Logic Interface Input Signals
--    SPI3_1_BrstAddr       : Write Logic Address 
--    SPI3_1_BrstData       : Write Logic Data 
--    SPI3_1_BrstMod        : Write Logic MOD 
--    SPI3_1_BrstSOP        : Write Logic Start of Packet 
--    SPI3_1_BrstEOP        : Write Logic End of Packet  
--    SPI3_1_BrstErr        : Write Logic Error  
--    SPI3_1_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
--                            (active low)
--
-- SPI3 Number 2 Write Logic Interface Input Signals
--    SPI3_2_BrstAddr       : Write Logic Address 
--    SPI3_2_BrstData       : Write Logic Data 
--    SPI3_2_BrstMod        : Write Logic MOD 
--    SPI3_2_BrstSOP        : Write Logic Start of Packet 
--    SPI3_2_BrstEOP        : Write Logic End of Packet  
--    SPI3_2_BrstErr        : Write Logic Error  
--    SPI3_2_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
--                            (active low)
--
-- SPI3 Number 3 Write Logic Interface Input Signals
--    SPI3_3_BrstAddr       : Write Logic Address 
--    SPI3_3_BrstData       : Write Logic Data 
--    SPI3_3_BrstMod        : Write Logic MOD 
--    SPI3_3_BrstSOP        : Write Logic Start of Packet 
--    SPI3_3_BrstEOP        : Write Logic End of Packet  
--    SPI3_3_BrstErr        : Write Logic Error  
--    SPI3_3_BrstRdEn_n     : Write Logic Data will be valid on next clock cycle
--                            (active low)
--
-- SPI3 Arbiter Interface Input Signals
--    SPI3_0_BrstWrEn_n     : Burst Write Enable for SPI3 0 (active low)
--    SPI3_1_BrstWrEn_n     : Burst Write Enable for SPI3 1 (active low)
--    SPI3_2_BrstWrEn_n     : Burst Write Enable for SPI3 2 (active low)
--    SPI3_3_BrstWrEn_n     : Burst Write Enable for SPI3 3 (active low)
--
-- SPI4 Interface Output Signals
--    SPI3_WrAddr           : SPI4 Interface Address 
--    SPI3_WrData           : SPI4 Interface Data 
--    SPI3_WrMod            : SPI4 Interface MOD 
--    SPI3_WrSOP            : SPI4 Interface Start of Packet 
--    SPI3_WrEOP            : SPI4 Interface End of Packet  
--    SPI3_WrErr            : SPI4 Interface Error  
--    SPI3_WrEn_n           : SPI4 Interface Write Enable (active low)  
--
-- SPI3 Burst Storage Interface Output Signals
--    SPI3_Full_n           : Burst Logic Full (active low) 
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi3_to_spi4_write Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi3_to_spi4_write is

  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------
  signal Sel_SPI3Addr   : std_logic_vector(7 downto 0);
  signal Sel_SPI3Data   : std_logic_vector(63 downto 0);
  signal Sel_SPI3Mod    : std_logic_vector(2 downto 0);
  signal Sel_SPI3SOP    : std_logic;
  signal Sel_SPI3EOP    : std_logic;
  signal Sel_SPI3Err    : std_logic;
--  signal SPI4WrEn       : std_logic;    
begin

  
  ------------------------------------------------------------------------------
  -- Process to register selected Channels Burst Storage Outputs
  --
  ------------------------------------------------------------------------------
  spi3reg : process ( Reset_n, BridgeClk )
  begin
    if( Reset_n = '0' ) then
      Sel_SPI3Addr <= (others => '0') after TFF;
      Sel_SPI3Data <= (others => '0') after TFF;
      Sel_SPI3Mod  <= "000" after TFF;
      Sel_SPI3SOP  <= '0' after TFF;
      Sel_SPI3EOP  <= '0' after TFF;
      Sel_SPI3Err  <= '0' after TFF;
      --SPI4WrEn     <= '0' after TFF;
    elsif( BridgeClk = '1' and BridgeClk'event ) then
      --SPI4WrEn   <=  ArbiterWrEn;

      case ArbiterRdChan is
        when "00" =>
            Sel_SPI3Addr <= conv_std_logic_vector(0, 8);
            Sel_SPI3Data <= SPI3_0_BrstData;
            Sel_SPI3Mod  <= SPI3_0_BrstMod;
            Sel_SPI3SOP  <= SPI3_0_BrstSOP;
            Sel_SPI3EOP  <= SPI3_0_BrstEOP;
            Sel_SPI3Err  <= SPI3_0_BrstErr;
            
        when "01" =>
            Sel_SPI3Addr <= conv_std_logic_vector(1, 8);
            Sel_SPI3Data <= SPI3_1_BrstData;
            Sel_SPI3Mod  <= SPI3_1_BrstMod;
            Sel_SPI3SOP  <= SPI3_1_BrstSOP;
            Sel_SPI3EOP  <= SPI3_1_BrstEOP;
            Sel_SPI3Err  <= SPI3_1_BrstErr;
            
        when "10" =>
            Sel_SPI3Addr <= conv_std_logic_vector(2, 8);
            Sel_SPI3Data <= SPI3_2_BrstData;
            Sel_SPI3Mod  <= SPI3_2_BrstMod;
            Sel_SPI3SOP  <= SPI3_2_BrstSOP;
            Sel_SPI3EOP  <= SPI3_2_BrstEOP;
            Sel_SPI3Err  <= SPI3_2_BrstErr;

        when "11" =>
            Sel_SPI3Addr <= conv_std_logic_vector(3, 8);
            Sel_SPI3Data <= SPI3_3_BrstData;
            Sel_SPI3Mod  <= SPI3_3_BrstMod;
            Sel_SPI3SOP  <= SPI3_3_BrstSOP;
            Sel_SPI3EOP  <= SPI3_3_BrstEOP;
            Sel_SPI3Err  <= SPI3_3_BrstErr;
        when others => null;
      end case;
    end if;  
  end process ;

  ------------------------------------------------------------------------------
  -- Combinatorial Output Signal Assignments
  ------------------------------------------------------------------------------
  SPI4_WrEn_n <= not ArbiterWrEn;
  SPI4_WrAddr <= Sel_SPI3Addr;
  SPI4_WrData <= Sel_SPI3Data;
  SPI4_WrMod  <= Sel_SPI3Mod;
  SPI4_WrSOP  <= Sel_SPI3SOP;
  SPI4_WrEOP  <= Sel_SPI3EOP;
  SPI4_WrErr  <= Sel_SPI3Err;
  
end xilinx;
