-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_spi3_flow_control.vhd
--     Version:  1.0
--        Date:  April 11, 2003
--
-- Description:  The purpose of the spi4_to_spi3_flow_control.vhd file is to 
--               drive the SPI4.2 Core Status Interface with the proper flow
--               control information depending on signals received from the
--               SPI-3 cores and the burst storage FIFOs.
--
--      Author:  IP Solutions Division
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi4_to_spi3_flow_control is
  port (
    -- System signals
    Reset_n                   : in  std_logic;
    SPI4_Clk                  : in  std_logic;

    -- SPI3 Core Status Input Signals
    SPI3_0_dst_rdy       : in  std_logic;
    SPI3_1_dst_rdy       : in  std_logic;
    SPI3_2_dst_rdy       : in  std_logic;
    SPI3_3_dst_rdy       : in  std_logic;
    
    -- Burst Storage Status Signals
    SPI3_0_Brst_HalfFull : in  std_logic;
    SPI3_1_Brst_HalfFull : in  std_logic;
    SPI3_2_Brst_HalfFull : in  std_logic;
    SPI3_3_Brst_HalfFull : in  std_logic;

    -- SPI4 Status Output Signals
    SPI4_StatWrEn_n           : out std_logic;
    SPI4_Stat                 : out std_logic_vector(7 downto 0)
    );
end spi4_to_spi3_flow_control;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                   : System Level Reset (active low)
--    SPI4_Clk                  : SPI4 Interface Clock 
--
-- SPI3 Core Status Input Signals
--    SPI3_0_dst_rdy       : SPI3 0 is available (active high)
--    SPI3_1_dst_rdy       : SPI3 1 is available (active high) 
--    SPI3_2_dst_rdy       : SPI3 2 is available (active high) 
--    SPI3_3_dst_rdy       : SPI3 3 is available (active high)
--
-- Burst Storage Status Signals
--    SPI3_0_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
--    SPI3_1_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
--    SPI3_2_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
--    SPI3_3_Brst_HalfFull      : Burst Storage FIFO 0 is half full 
--
-- SPI4 Status Output Signals
--    SPI4_StatWrEn_n           : SPI4 Status Write Enable (active low)
--    SPI4_Stat                 : SPI4 Status - 2-bits per channel
--                                (7:6) = Channel 3, (1:0) = Channel 0
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi4_to_spi3_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi4_to_spi3_flow_control is

begin

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  write_status: process ( Reset_n, SPI4_Clk )
  begin
    if ( Reset_n = '0' ) then
    
      SPI4_Stat             <= "10101010";
    
    elsif ( SPI4_Clk = '1' and SPI4_Clk'event ) then
    
      -- "00" = starving, "10" = satisfied, "01" = hungry, "11" = framing
      -- If SPI 3 source core is Almost Full or burst FIFO is half full,
      -- then send satisified on the Full channel. Else, send starving.

      if (SPI3_0_dst_rdy = '0' or SPI3_0_Brst_HalfFull='1') then
        SPI4_Stat(1 downto 0) <= "10";
      else
        SPI4_Stat(1 downto 0) <= "00";
      end if;
      
      if (SPI3_1_dst_rdy = '0' or SPI3_1_Brst_HalfFull='1') then
        SPI4_Stat(3 downto 2) <= "10";
      else
        SPI4_Stat(3 downto 2) <= "00";
      end if;
      
      if (SPI3_2_dst_rdy = '0' or SPI3_2_Brst_HalfFull='1') then
        SPI4_Stat(5 downto 4) <= "10";
      else
        SPI4_Stat(5 downto 4) <= "00";
      end if;
      
      if (SPI3_3_dst_rdy = '0' or SPI3_3_Brst_HalfFull='1') then
        SPI4_Stat(7 downto 6) <= "10";
      else
        SPI4_Stat(7 downto 6) <= "00";
      end if;
		
--      if (SPI3_0_AlmostFull_n = '0' or SPI3_0_Brst_HalfFull='1') then
--        SPI4_Stat(1 downto 0) <= "10";
--      else
--        SPI4_Stat(1 downto 0) <= "00";
--      end if;
--      
--      if (SPI3_1_AlmostFull_n = '0' or SPI3_1_Brst_HalfFull='1') then
--        SPI4_Stat(3 downto 2) <= "10";
--      else
--        SPI4_Stat(3 downto 2) <= "00";
--      end if;
--      
--      if (SPI3_2_AlmostFull_n = '0' or SPI3_2_Brst_HalfFull='1') then
--        SPI4_Stat(5 downto 4) <= "10";
--      else
--        SPI4_Stat(5 downto 4) <= "00";
--      end if;
--      
--      if (SPI3_3_AlmostFull_n = '0' or SPI3_3_Brst_HalfFull='1') then
--        SPI4_Stat(7 downto 6) <= "10";
--      else
--        SPI4_Stat(7 downto 6) <= "00";
--      end if;

    end if;  
  end process write_status;
  
  SPI4_StatWrEn_n <= '0';

end xilinx;
