-------------------------------------------------------------------------------
--
--   File Name:  spi4_to_spi3_read.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi4_to_spi3_read.vhd file is to provide 
--               the SPI-4.2 core read logic. This logic will read data from 
--               the SPI-4.2 Core as long as the SPI4.2 Core is not empty.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi4_to_spi3_read is
  port (
    -- System signals
    Reset_n               : in  std_logic;
    SPI4_Clk              : in  std_logic;

    -- SPI4 Core Interface Input Signals
    SPI4_FFAddr           : in  std_logic_vector(7 downto 0);
    SPI4_FFData           : in  std_logic_vector(63 downto 0);
    SPI4_FFMod            : in  std_logic_vector(2 downto 0);
    SPI4_FFSOP            : in  std_logic;
    SPI4_FFEOP            : in  std_logic;
    SPI4_FFErr            : in  std_logic;
    SPI4_FFValid          : in  std_logic;
    SPI4_FFPayloadErr     : in  std_logic;
    SPI4_FFDIP4Err        : in  std_logic;
	 SPI4_FFBurstErr		  : in  std_logic;
	 SPI4_FFPayloadDIP4	  : in  std_logic;
    SPI4_FFEmpty_n        : in  std_logic;
    SPI4_FFAlmostEmpty_n  : in  std_logic;

    -- SPI4 Core Interface Output Signals
    SPI4_FFRdEn_n         : out std_logic;
    
    -- SPI4 Write Logic Interface Output Signals
    SPI4_Addr             : out std_logic_vector(7 downto 0);
    SPI4_Data             : out std_logic_vector(63 downto 0);
    SPI4_Mod              : out std_logic_vector(2 downto 0);
    SPI4_SOP              : out std_logic;
    SPI4_EOP              : out std_logic;
    SPI4_Err              : out std_logic;
    SPI4_Valid            : out std_logic
    );
end spi4_to_spi3_read;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n               : System Level Reset (active low)
--    SPI4_Clk              : SPI4 Interface Clock 

-- SPI4 Core Interface Input Signals
--    SPI4_FFAddr           : SPI4 Interface Address
--    SPI4_FFData           : SPI4 Interface Data
--    SPI4_FFMod            : SPI4 Interface MOD
--    SPI4_FFSOP            : SPI4 Interface Start of Packet 
--    SPI4_FFEOP            : SPI4 Interface End of Packet 
--    SPI4_FFErr            : SPI4 Interface Error 
--    SPI4_FFValid          : SPI4 Interface Data is Valid 
--    SPI4_FFPayloadErr     : SPI4 Interface Payload Error (current data was
--                            not preceded by a payload control word
--    SPI4_FFDIP4Err        : SPI4 Interface DIP-4 Error was detected on current
--                            end of burst (either EOP or N-credit boundary)
--    SPI4_FFEmpty_n        : SPI4 Interface is Empty (active low) 
--    SPI4_FFAlmostEmpty_n  : SPI4 Interface is Almost Empty (active low)

-- SPI4 Core Interface Output Signals
--    SPI4_FFRdEn_n         : SPI4 Interface Read Enable (active low)

-- SPI4 Write Logic Interface Output Signals
--    SPI4_Addr             : Write Logic Address 
--    SPI4_Data             : Write Logic Data 
--    SPI4_Mod              : Write Logic MOD 
--    SPI4_SOP              : Write Logic Start of Packet 
--    SPI4_EOP              : Write Logic End of Packet  
--    SPI4_Err              : Write Logic Error  
--    SPI4_Valid            : Write Logic Data is Valid  
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi4_to_spi3_read Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi4_to_spi3_read is

  ------------------------------------------------------------------------------
  -- Constant Declarations
  ------------------------------------------------------------------------------

  -----------------------------------------------------------------------------
  -- State Machine Variable and Type Declarations
  -- StateType: 
  --    IDLE             - Reset and idle state.
  --    REQUEST          - A data transfer request. This indicates that the
  --                       loopback is enabled, the Receive FIFO has data, and
  --                       the Transmit FIFO can accept data.
  --    TRANSFER         - Data is transferred in this state. Remain in this
  --                       state until the loopback is disabled,
  --                       the Receive FIFO
  --                       doesn't have any more data, or the
  --                       Transmit FIFO cannot
  --                       accept any more data.
  --    ONE_TRANSFER     - Indicates that the Receive FIFO has only one word
  --                       available for reading.
  --
  -- Variables:
  --    State   - holds current state of state machine
  -----------------------------------------------------------------------------
  type StateType is (IDLE, REQUEST, TRANSFER, ONE_TRANSFER);
  signal State : StateType;

  -----------------------------------------------------------------------------
  -- Signal Declarations
  -- OneWordLeft - Indicates that only one word is left in the FIFO, and a 
  --               single read should be performed
  -----------------------------------------------------------------------------
  signal OneWordLeft            : std_logic;
  signal SPI4_FFEmpty_n_r       : std_logic;
  signal SPI4_FFAlmostEmpty_n_r : std_logic;

begin

  -----------------------------------------------------------------------------
  -- Drive out the inputs from the Receive FIFO.
  -----------------------------------------------------------------------------
  SPI4_Data   <= SPI4_FFData;
  SPI4_Addr   <= SPI4_FFAddr;
  SPI4_Mod    <= SPI4_FFMod;
  SPI4_SOP    <= SPI4_FFSOP;
  SPI4_EOP    <= SPI4_FFEOP;
  SPI4_Err    <= SPI4_FFErr;
  SPI4_Valid  <= SPI4_FFValid;

  -----------------------------------------------------------------------------
  -- Determine if only one word is left in the FIFO. This is true only if 
  -- Empty_n was asserted on the previous cycle, and AlmostEmpty_n is not 
  -- asserted on the current cycle.
  -----------------------------------------------------------------------------
  reg_empty: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n = '0') then
      SPI4_FFEmpty_n_r       <= '0' ;
      SPI4_FFAlmostEmpty_n_r <= '0' ;
    elsif (SPI4_Clk'event and SPI4_Clk = '1') then
      SPI4_FFEmpty_n_r       <= SPI4_FFEmpty_n ;
      SPI4_FFAlmostEmpty_n_r <= SPI4_FFAlmostEmpty_n ;
    end if;
  end process reg_empty;

  -- In order for only one word left to be true, then not AE_n and E_n needs
  -- to be true for at least two cycles. This prevents flagging an incorrect
  -- one word left indication as the FIFO is transitioning from E to AE.
  one_left: process (Reset_n, SPI4_Clk)
  begin
    if (Reset_n = '0') then
      OneWordLeft <= '0' ;
    elsif (SPI4_Clk'event and SPI4_Clk = '1') then
      OneWordLeft <= ((not SPI4_FFAlmostEmpty_n and SPI4_FFEmpty_n) and
                      (not SPI4_FFAlmostEmpty_n_r and SPI4_FFEmpty_n_r)) 
                      ;
    end if;
  end process one_left;

  -----------------------------------------------------------------------------
  -- State Machine Process
  -----------------------------------------------------------------------------
  state_machine : process ( Reset_n, SPI4_Clk )
  begin
    if(Reset_n = '0') then
      State         <= IDLE;
      SPI4_FFRdEn_n <= '1' ;
    elsif (SPI4_Clk'event and SPI4_Clk = '1') then

      case State is
        -----------------------------------------------------------------------
        -- State 0: IDLE 
        -----------------------------------------------------------------------
        when IDLE =>
          --If the PL4 core is not almost empty, start reading
          if (SPI4_FFAlmostEmpty_n = '1') then
            State         <= REQUEST ;
            SPI4_FFRdEn_n <= '0' ;
          --If there is only one word left, only transfer one word out
          --of the PL4 core
          elsif (OneWordLeft = '1') then
            State         <= ONE_TRANSFER ;
            SPI4_FFRdEn_n <= '0' ;
          else
            State         <= IDLE ;
            SPI4_FFRdEn_n <= '1' ;
          end if;
        -----------------------------------------------------------------------
        -- State 1: REQUEST
        -----------------------------------------------------------------------
        when REQUEST =>
          State         <= TRANSFER ;
          SPI4_FFRdEn_n <= '0' ;
        -----------------------------------------------------------------------
        -- State 2: TRANSFER
        -----------------------------------------------------------------------
        when TRANSFER =>
          if (SPI4_FFAlmostEmpty_n = '1') then
            State         <= TRANSFER ;
            SPI4_FFRdEn_n <= '0' ;
          else
            State         <= IDLE ;
            SPI4_FFRdEn_n <= '1' ;
          end if;
        -----------------------------------------------------------------------
        -- State 3: ONE_TRANSFER
        -----------------------------------------------------------------------
        when ONE_TRANSFER =>
          State         <= IDLE ;
          SPI4_FFRdEn_n <= '1' ;
        -----------------------------------------------------------------------
        -- Error and Default States
        -----------------------------------------------------------------------
        when OTHERS =>
          State         <= IDLE ;
          SPI4_FFRdEn_n <= '1' ;
      end case;
    end if;
  end process state_machine;

end xilinx;
