-------------------------------------------------------------------------------
--
--   File Name:  spi_clk_startup.vhd
--     Version:  1.0
--        Date:  June 11, 2007
--
-- Description:  The purpose of the spi_clk_startup.vhd file is to set the
--               proper startup sequence for the DCMs. This will allow proper
--               initialization of the clocking scheme for the entire system.
--
--      Author:  Zhe Xia
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Library Declarations
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.conv_std_logic_vector;
use ieee.std_logic_unsigned.all;

library work;
use work.spi_pkg.all;

--------------------------------------------------------------------------------
-- Entity Declaration
--------------------------------------------------------------------------------

entity spi_clk_startup is
  port (
    -- System signals
    Reset_n                   : in  std_logic;
    SPI_Clk                   : in  std_logic;

    -- SPI4.2 Sink DCM Status
    SPI4_DcmReset_RDClk     : out std_logic;
    SPI4_Locked_RDClk       : in  std_logic;

    -- SPI4.2 Source DCM Status
    SPI4_DcmReset_TDClk     : out std_logic;
    SPI4_Locked_TDClk       : in  std_logic;
    SPI4_DcmReset_TSClk     : out std_logic;
    SPI4_Locked_TSClk       : in  std_logic;

    PL4Rst_n                : out std_logic;
    
    -- Calendar Programming
    SnkStatClk           : in  std_logic;
    SrcStatClk           : in  std_logic;
    CalProg              : out std_logic;
    SnkCalProgDone       : in  std_logic;
    SrcCalProgDone       : in  std_logic;
    
    -- Sink and Source Core enables
    SnkEn              : out std_logic;
    SrcEn              : out std_logic;
    
    -- Dynamic phase aligning signals
    SnkFFClk           : in  std_logic;
    PhaseAlignRequest  : out std_logic;
    PhaseAlignComplete : in  std_logic;
    SnkOof             : in  std_logic
    
    );
end spi_clk_startup;

--------------------------------------------------------------------------------
-- Definition of Ports:
--------------------------------------------------------------------------------
-- System signals
--    Reset_n                 : Input Reset (active low)
--    SPI_Clk                 : SPI Interface Clock 
--    SPI4_DcmReset_RDClk     : RDClk DCM Reset (active high)
--    SPI4_Locked_RDClk       : RDClk DCM Locked signal (active high)
--    SPI4_DcmReset_TDClk     : TDClk DCM Reset (active high)
--    SPI4_Locked_TDClk       : TDClk DCM Locked signal (active high)
--    SPI4_DcmReset_TSClk     : TSClk DCM Reset (active high)
--    SPI4_Locked_TSClk       : TSClk DCM Locked signal (active high)
--    PL4Rst_n                : System Level Reset indicating all DCMs are
--                              locked
--    SnkStatClk              : Clock used for programming the Snk calendar
--    SrcStatClk              : Clock used for programming the Src calendar
--    SnkCalProgDone          : Signal indicating the Snk calendar programming
--                              logic has completed programming
--    SrcCalProgDone          : Signal indicating the Src calendar programming
--                              logic has completed programming
--    SnkEn                   : Enable signal for the SPI-4.2 sink core
--    SrcEn                   : Enable signal for the SPI-4.2 source core
--    SnkFFClk                : Clock used for talking to the user side of the
--                              SPI-4.2 sink core
--    PhaseAlignRequest       : Signal indicating that the SPI-4.2 sink core
--                              should re-align the data with the clock
--    PhaseAlignComplete      : Signal indicated the SPI-4.2 sink core has
--                              complete re-aligning the data with the clock
--    SnkOof                  : Signal indicating the data and clock are mis-
--                              aligned in the SPI-4.2 sink core
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- spi_clk_startup Architecture
--------------------------------------------------------------------------------
architecture xilinx of spi_clk_startup is

  ------------------------------------------------------------------------------
  -- Constant Declarations
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- Signal Declarations
  ------------------------------------------------------------------------------
  type FSM_STATE is (IDLE,
                 TDCLK_RST,
                 TDCLK_LCK,
                 RDCLK_RST,
                 RDCLK_LCK,
                 TSCLK_RST,
                 TSCLK_LCK,
                 RELEASE_RST);
                 
  type snk_seq_state is (PA_IDLE,
                         PA_REQ,
                         PA_CMPLT);

  signal state             : FSM_STATE;
  signal CountClear        : std_logic;
  signal Count             : std_logic_vector(8 downto 0);
  signal state_snk         : snk_seq_state;
  signal wait_count        : integer range 0 to 100 := 0;
  signal PL4Rst_n_int      : std_logic;
  signal SrcEn_i           : std_logic;
  signal SnkEn_i           : std_logic;

begin

counter: process (Reset_n, SPI_Clk)
begin
  if (Reset_n = '0') then
    Count <= (others => '0');
  elsif (SPI_Clk'event and SPI_Clk = '1') then
    if (CountClear = '1') then
      Count <= (others => '0');
    else
      Count <= Count + 1;
    end if;
  end if;
end process counter;

rst_fsm: process (Reset_n, SPI_Clk)
begin
  if (Reset_n = '0') then
    state               <= IDLE;
    SPI4_DcmReset_TDClk <= '0';
    SPI4_DcmReset_RDClk <= '0';
    SPI4_DcmReset_TSClk <= '0';
    CountClear          <= '1';
    PL4Rst_n_int        <= '0';
  elsif (SPI_Clk'event and SPI_Clk = '1') then
    case state is
      
      when IDLE =>
        state               <= TDCLK_RST;
        SPI4_DcmReset_TDClk <= '1';
        SPI4_DcmReset_RDClk <= '0';
        SPI4_DcmReset_TSClk <= '0';
        CountClear          <= '1';
        PL4Rst_n_int        <= '0';

      when TDCLK_RST =>
        -- Expand DCM Reset pulse duration to 8 clock cycles
        if (Count = conv_std_logic_vector( 8, Count'length)) then
          state               <= TDCLK_LCK;
          SPI4_DcmReset_TDClk <= '0';
        else
          state               <= TDCLK_RST;
          SPI4_DcmReset_TDClk <= '1';
        end if;
        SPI4_DcmReset_RDClk <= '0';
        SPI4_DcmReset_TSClk <= '0';
        CountClear          <= '0';
        PL4Rst_n_int        <= '0';

      when TDCLK_LCK =>
        -- If TDCLK DCM is locked, then go to next state
        if (SPI4_Locked_TDClk = '1') then
          state               <= RDCLK_RST;
          SPI4_DcmReset_RDClk <= '1';
          SPI4_DcmReset_TDClk <= '0';
          CountClear          <= '1';
        -- If counter has wrapper all the way around (256 cycles), then reset
        -- the DCM again and go back to TDCLK_RST.
        elsif (Count = conv_std_logic_vector( 8, Count'length)) then
          state               <= TDCLK_RST;
          SPI4_DcmReset_TDClk <= '1';
          SPI4_DcmReset_RDClk <= '0';
          CountClear          <= '1';
        -- Else wait for the DCM to lock or the counter to expire
        else
          state               <= TDCLK_LCK;
          SPI4_DcmReset_TDClk <= '0';
          SPI4_DcmReset_RDClk <= '0';
          CountClear          <= '0';
        end if;
        SPI4_DcmReset_TSClk <= '0';
        PL4Rst_n_int        <= '0';

        when RDCLK_RST =>
        -- Expand DCM Reset pulse duration to 8 clock cycles
        if (Count = conv_std_logic_vector( 8, Count'length)) then
          state               <= RDCLK_LCK;
          SPI4_DcmReset_RDClk <= '0';
        else
          state               <= RDCLK_RST;
          SPI4_DcmReset_RDClk <= '1';
        end if;
        SPI4_DcmReset_TDClk <= '0';
        SPI4_DcmReset_TSClk <= '0';
        CountClear          <= '0';
        PL4Rst_n_int        <= '0';

      when RDCLK_LCK =>
        -- If RDCLK DCM is locked, then go to next state
        if (SPI4_Locked_RDClk = '1') then
          state               <= TSCLK_RST;
          SPI4_DcmReset_RDClk <= '0';
          SPI4_DcmReset_TSClk <= '1';
          CountClear          <= '1';
        -- If counter has wrapper all the way around (256 cycles), then reset
        -- the DCM again and go back to RDCLK_RST.
        elsif (Count = conv_std_logic_vector( 8, Count'length)) then
          state               <= RDCLK_RST;
          SPI4_DcmReset_RDClk <= '1';
          SPI4_DcmReset_TSClk <= '0';
          CountClear          <= '1';
        -- Else wait for the DCM to lock or the counter to expire
        else
          state               <= RDCLK_LCK;
          SPI4_DcmReset_RDClk <= '0';
          SPI4_DcmReset_TSClk <= '0';
          CountClear          <= '0';
        end if;
        SPI4_DcmReset_TDClk <= '0';
        PL4Rst_n_int        <= '0';

      when TSCLK_RST =>
        -- Expand DCM Reset pulse duration to 8 clock cycles
        if (Count = conv_std_logic_vector( 8, Count'length)) then
          state               <= TSCLK_LCK;
          SPI4_DcmReset_TSClk <= '0';
        else
          state               <= TSCLK_RST;
          SPI4_DcmReset_TSClk <= '1';
        end if;
        SPI4_DcmReset_TDClk <= '0';
        SPI4_DcmReset_RDClk <= '0';
        CountClear          <= '0';
        PL4Rst_n_int        <= '0';

      when TSCLK_LCK =>
        -- If TSCLK DCM is locked, then go to next state
        if (SPI4_Locked_TSClk = '1') then
          state               <= RELEASE_RST;
          SPI4_DcmReset_TSClk <= '0';
          CountClear          <= '1';
        -- If counter has wrapper all the way around (256 cycles), then reset
        -- the DCM again and go back to TSCLK_RST.
        elsif (Count = conv_std_logic_vector( 8, Count'length)) then
          state               <= TSCLK_RST;
          SPI4_DcmReset_TSClk <= '1';
          CountClear          <= '1';
        -- Else wait for the DCM to lock or the counter to expire
        else
          state               <= TSCLK_LCK;
          SPI4_DcmReset_TSClk <= '0';
          CountClear          <= '0';
        end if;
        SPI4_DcmReset_TDClk <= '0';
        SPI4_DcmReset_RDClk <= '0';
        PL4Rst_n_int        <= '0';

      when RELEASE_RST =>
        SPI4_DcmReset_TDClk <= '0';
        SPI4_DcmReset_RDClk <= '0';
        SPI4_DcmReset_TSClk <= '0';
        CountClear          <= '1';      
        PL4Rst_n_int        <= '1';
      
      when OTHERS =>
        state               <= IDLE;
        SPI4_DcmReset_TDClk <= '0';
        SPI4_DcmReset_RDClk <= '0';
        SPI4_DcmReset_TSClk <= '0' ;
        CountClear          <= '1';
        PL4Rst_n_int        <= '0';

    end case;
  end if;
end process rst_fsm;

src_en : process (PL4Rst_n_int, SrcStatClk)
begin
  if (PL4Rst_n_int = '0') then
    SrcEn_i <= '0';    
  elsif (SrcStatClk'event and SrcStatClk='1') then
    SrcEn_i <= SrcCalProgDone;    
  end if;
end process src_en;

snk_en : process (PL4Rst_n_int, SnkStatClk)
begin
  if (PL4Rst_n_int = '0') then
    SnkEn_i            <= '0';
  elsif (SnkStatClk'event and SnkStatClk = '1') then
    SnkEn_i <= SnkCalProgDone;
  end if;
end process snk_en;

snk_fsm : process (Reset_n, SnkFFClk)
begin
  if (Reset_n = '0') then
    PhaseAlignRequest  <= '0';
    state_snk          <= PA_IDLE;
    wait_count         <= 0;
    
  elsif (SnkFFClk'event and SnkFFClk = '1') then
      
    case state_snk is
      -- PA_IDLE State : wait for completion of DCM resets
      when PA_IDLE =>
        if (SnkOof='1' and SnkEn_i='1') then
          PhaseAlignRequest  <= '0';
          state_snk          <= PA_REQ;
          wait_count         <= 0;
        else
          PhaseAlignRequest  <= '0';
          state_snk          <= PA_IDLE;
          wait_count         <= 0;
        end if;

      -- PA_REQ state : When using dynamic alignment PhaseAlignRequest
      -- signal is triggered in this state
      when PA_REQ =>
        PhaseAlignRequest  <= '1';
        state_snk          <= PA_CMPLT;
        wait_count         <= 0;
          
      -- PA_CMPLT state : Waits for the PhaseAlignComplete signal.
      -- If the signal is not received within 30 cycle, it returns to
      -- the PA_REQ state.
      when PA_CMPLT =>
        if (wait_count  = 100) then
          PhaseAlignRequest  <= '0';
          wait_count         <= 0;
          if (PhaseAlignComplete = '1') then
            state_snk    <= PA_IDLE;
          else
            state_snk    <= PA_REQ;
          end if;
        else
          PhaseAlignRequest  <= '0';
          state_snk          <= PA_CMPLT;
          wait_count         <= wait_count  + 1;
        end if;
        
    end case;
  end if;
end process snk_fsm;

PL4Rst_n <= PL4Rst_n_int;
CalProg <= PL4Rst_n_int;
SrcEn <= SrcEn_i;
SnkEn <= SnkEn_i;

end xilinx;
