/******************************************************************************
*
*   File Name:  spi3_emulator_phy.vhd
*     Version:  2.0
*        Date:  October 15, 2004 
*
* Description:  This file simulates a single-channel POS-PHY Level 3 Physical
*               Layer. It continuously transmits an eight word packet with
*               the appropriate Start Of Packet and End Of Packet Signals to
*               different channels.
*
*      Author:  IP Solutions Division
*     Company:  Xilinx Inc.
*
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2003 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/


`timescale 1 ps / 1 ps

/*******************************************************************************
 * Module Declarations
 ******************************************************************************/

module spi3_emulator_phy (
  SPI3_EMUL_RST_n,

  SPI3_RFClk,
  SPI3_RENB_n,
  SPI3_RDAT,
  SPI3_RSOP,
  SPI3_REOP,
  SPI3_RMOD,
  SPI3_RPRTY,
  SPI3_RERR,
  SPI3_RVAL,
  SPI3_RSX,
  
  SPI3_TFClk,
  SPI3_TENB_n,
  SPI3_TDAT,
  SPI3_TSOP,
  SPI3_TEOP,
  SPI3_TMOD,
  SPI3_TPRTY,
  SPI3_TERR,
  SPI3_TSX,
  SPI3_PTPA,
  SPI3_TADR
  );



  /****************************************************************************
  * I/O Direction Declarations
  ****************************************************************************/
  input         SPI3_EMUL_RST_n;

  input         SPI3_RFClk;
  input         SPI3_RENB_n;
  output [31:0] SPI3_RDAT;
  output        SPI3_RSOP;
  output        SPI3_REOP;
  output  [1:0] SPI3_RMOD;
  output        SPI3_RPRTY;
  output        SPI3_RERR;
  output        SPI3_RVAL;
  output        SPI3_RSX;

  input         SPI3_TFClk;
  input         SPI3_TENB_n;
  input  [31:0] SPI3_TDAT;
  input         SPI3_TSOP;
  input         SPI3_TEOP;
  input   [1:0] SPI3_TMOD;
  input         SPI3_TPRTY;
  input         SPI3_TERR;
  input         SPI3_TSX;
  output        SPI3_PTPA;
  input  [7:0]  SPI3_TADR;

`include "spi3_phy_para.v"

/****************************************************************************
  ** Parameter Declarations
  ****************************************************************************/


//  parameter DataOffset               = 0; 
//
//  //There are n channels that are numbered from 0 to n-1.  This constant
//  // represents the channel with the largest number. 
//  parameter TopChannel = 0;   // Only have a single channel core.
//
//  // NumOfWords is the number of 32 bit words that are transferred per packet.
//  // Modify this constant to change the number of words per packet.
//  parameter NumOfWords = 8;
//
//  // AddressWord is the cycle where RSX is asserted and the in-band address is
//  // on the SPI3_RDat bus.
//  parameter AddressWord = DataOffset;
//
//  // FirstWord is the cycle where SPI3_RSOP is asserted, it is the first word
//  // of the packet
//  parameter FirstWord = 1;
//
//  // LastWord is the cycle where SPI3_REOP is asserted, it is the last word of
//  // the packet
//  parameter LastWord = AddressWord + NumOfWords;
//
//  // DelayLength is used to limit the SPI3_RErr signal so that it does not
//  // occur in every cycle
//  parameter DelayLength = 4;
//
//  // Constant time delay to model the clock to Q delay of registers.
//  parameter NS = 1000;
//  parameter TFF = 2 * NS;


  /****************************************************************************
  * I/O Type Declarations
  ****************************************************************************/

  wire SPI3_EMUL_RST_n;

  wire        SPI3_RFClk;
  wire        SPI3_RENB_n;
  wire [31:0] SPI3_RDAT;
  wire        SPI3_RSOP;
  wire        SPI3_REOP;
  wire  [1:0] SPI3_RMOD;
  wire        SPI3_RPRTY;
  wire        SPI3_RVAL;
  wire        SPI3_RERR;
  wire        SPI3_RSX;

  wire        SPI3_TFClk;
  wire        SPI3_TENB_n;
  wire [31:0] SPI3_TDAT;
  wire        SPI3_TSOP;
  wire        SPI3_TEOP;
  wire  [1:0] SPI3_TMOD;
  wire        SPI3_TPRTY;
  wire        SPI3_TERR;
  wire        SPI3_TSX;
  wire        SPI3_PTPA;
  wire  [7:0] SPI3_TADR;


  /****************************************************************************
  * Signal Declarations
  ****************************************************************************/
  
  reg ptpa_i;
  reg ptpa_d0;

  reg [31:0] rdat_i;
  reg [1:0] rmod_i;
  reg rprty_i;
  reg rval_i;
  reg rsop_i;
  reg reop_i;
  reg rerr_i;
  reg rsx_i;

  reg [3:0] word_num;
  reg       chan_num;
  reg [2:0] delay_num;
  reg [6:0] trans_num;
  reg       tchan_num;

  initial
  begin
    ptpa_i      = 0;
    ptpa_d0     = 0;
    chan_num    = 0;
    tchan_num   = 0;
    word_num = AddressWord;
  end

  /*****************************************************************************
  ** The following always block Handles the SPI3_RSOP Signal
  *****************************************************************************/
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      rsop_i <= #TFF 0;
     end
    else
     begin
      /*************************************************************************
      ** SPI3_RENB_n pin is active low.  When the SPI3_RENB_n pin is active the
      ** Physical Layer does the appropriate operations, otherwise it
      ** holds its previous state
      *************************************************************************/
      if (SPI3_RENB_n== 0)
       begin
        /***********************************************************************
        ** When the Physical Layer is ready to transfer the first
        ** word of the packet, the Physical Layer will send out the
        ** RSOP pulse to tell the Link layer that a data packet is
        ** beginning to transfer.
        ***********************************************************************/
        if (word_num==FirstWord)
         begin
          rsop_i <= #TFF 1;
         end
        else
         begin
          rsop_i <= #TFF 0;
         end
       end // if (SPI3_RENB_n=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
  /*****************************************************************************
   * The following always block Handles the SPI3_REOP Signal
   ****************************************************************************/
   always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
    begin
     if (SPI3_EMUL_RST_n == 0)
      begin
       reop_i <= #TFF 0;
      end
     else
      begin
       /************************************************************************
       ** SPI3_RENB_n pin is active low.  When the RENB pin is active the
       ** Physical Layer does the appropriate operations, otherwise it
       ** holds its previous state
       ************************************************************************/
       if (SPI3_RENB_n==0)
        begin
         /**********************************************************************
         ** When the Physical Layer is ready to transfer the last word
         ** of the packet, the Physical Layer will send out the REOP
         ** pulse to tell the Link layer that a data packet transfer
         ** is complete.
         **********************************************************************/
         if (word_num==LastWord)
          begin
           reop_i <= #TFF 1;
          end
         else
          begin
           reop_i <= #TFF 0;
          end
        end // if (SPI3_RENB_n==1'b0)
      end // else: !if (SPI3_EMUL_RST_n == 1'b0)
    end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
  
  /*****************************************************************************
   * The following always block Handles the SPI3_RMOD Signal
   ****************************************************************************/
   always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
    begin
     if (SPI3_EMUL_RST_n == 0)
      begin
       rmod_i <= #TFF 0;
      end
     else
      begin
       /************************************************************************
       ** SPI3_RENB_n pin is active low.  When the SPI3_RENB_n pin is active the
       ** Physical Layer does the appropriate operations, otherwise it
       ** holds its previous state
       ************************************************************************/
       if (SPI3_RENB_n==0)
        begin
         /**********************************************************************
         ** Specify the number of bytes that are valid when in the
         ** last cycle of the packet transfer.  Otherwise set SPI3_RMOD to
         ** all zeros.  Will cycle through all possible values
         ** 00' to '11').
         **********************************************************************/
         if (word_num==LastWord)
          begin
           rmod_i <= #TFF delay_num % 4;
          end
         else
          begin
           rmod_i <= #TFF 0;
          end
        end // if (SPI3_RENB_n=='b0)
      end // else: !if (SPI3_EMUL_RST_n == 'b0)
    end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
  /*****************************************************************************
   * The following always block Handles the SPI3_RERR Signal
   ****************************************************************************/
   always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
    begin
     if (SPI3_EMUL_RST_n == 0)
      begin
       rerr_i <= #TFF 0;
      end
     else
      begin
       /************************************************************************
       ** SPI3_RENB_n pin is active low.  When the SPI3_RENB_i pin is active the
       ** Physical Layer does the appropriate operations, otherwise it
       ** holds its previous state
       ************************************************************************/
       if (SPI3_RENB_n==0)
        begin
         /**********************************************************************
         ** Specify that an error occured. This signal is activated on
         ** every nth packet (Where n is equal to Delay Length+1).  The
         ** RERR signal is only valid for the last cycle of the
         ** transfer.  In all other cases RERR is 0.
         **********************************************************************/
         if (word_num == LastWord)
          begin
           if (delay_num == DelayLength)
            begin
             rerr_i <= #TFF 1;
            end
           else
            begin
             rerr_i <= #TFF 0;
            end
          end // if (word_num == LastWord)
         else
          begin
           rerr_i <= #TFF 1'b0;
          end // else: !if (word_num == LastWord)
        end // if (SPI3_RENB_n=='b0)
      end // else: !if (SPI3_EMUL_RST_n == 'b0)
    end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
  /*****************************************************************************
  * The following always block Handles the SPI3_RDAT Signal
  *****************************************************************************/
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      rdat_i <= #TFF 0;
     end
    else
     begin
      /*************************************************************************
      ** SPI3_RENB_n pin is active low.  When the SPI3_RENB_n pin is active the
      ** Physical Layer does the appropriate operations, otherwise it
      ** holds its previous state
      *************************************************************************/
      if (SPI3_RENB_n==0)
       begin
 
        /***********************************************************************
        ** In the case that the Physical Layer is sending the in-band
        ** address on the RDAT bus, the format of the in-band address
        ** is as follows:
        ** 
        ** Bits 31-8: Ingored.  In this model they are all set to 0.
        ** 
        ** Bits 7-0: Contains the 8 bit encoding of the channel
        ** number that will be sending the next packet.
        ***********************************************************************/
        if (word_num == AddressWord)
         begin
          rdat_i <=  #TFF {24'b0, chan_num} ;
         end
        /***********************************************************************
        ** In the case that the Physical Layer is sending valid data,
        ** the data is formatted as follows:
        ** 
        ** Bits 31-24 : HEX coding for the channel number for which
        ** the transfer is taking place (ex. 01 is channel 1, 02 is
        ** channel 2)
        ** 
        ** Bits 23-16 : Concatination of the current word that is
        ** being transferred and the byte number (byte number is
        ** always 1 for this position)
        ** 
        ** Bits 15-8 : Concatination of the current word that is
        ** being transferred and the byte number (byte number is 
        ** always 2 for this position)
        ** 
        ** Bits 7-0 : Concatination of the current word that is being
        ** transferred and the byte number (byte number is always 3
        ** for this position)
        ** 
        **  (ex 01414243 is word 4 being transferred on channel 1)
        ***********************************************************************/
        else
         begin
          rdat_i <= #TFF {chan_num,
                          word_num, 4'h1,
                          word_num, 4'h2,
                          word_num, 4'h3
                         };
         end // else: !if (word_num == AddressWord)
       end // if (SPI3_RENB_n=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
  
 /******************************************************************************
  * The following always block Handles the SPI3_RVAL Signal
  *****************************************************************************/
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      rval_i <= #TFF 0;
     end
    else
     begin
      //RENB pin is active low.  When the RENB pin is active the
      //Physical Layer does the appropriate operations, otherwise it
      //holds its previous state
      if (SPI3_RENB_n==0)
       begin
        //When in the state where the in-band address will be put
        //on the RDAT bus, the RVAL signal must be low for the RSX
        //signal to be valid.
        if (word_num == AddressWord)
         begin
          rval_i <= #TFF 0;
         end
        else
         begin
          rval_i <= #TFF 1;
         end
       end // if (SPI3_RENB_n=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
 /******************************************************************************
  * The following always block Handles the SPI3_RSX Signal
  *****************************************************************************/
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      rsx_i <= #TFF 0;
     end
    else
     begin
      //RENB pin is active low.  When the RENB pin is active the
      //Physical Layer does the appropriate operations, otherwise it
      //holds its previous state
      if (SPI3_RENB_n==0)
       begin
        //When in the state where the in-band address will be put
        //on the RDAT bus, the Physical device will activate the RSX
        //signal.
        if (word_num == AddressWord)
         begin
          rsx_i <= #TFF 1;
         end
        else
         begin
          rsx_i <= #TFF 0;
         end
       end // if (SPI3_RENB=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
 /******************************************************************************
  * The following always block Handles the chan_num counter
  *****************************************************************************/
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      chan_num <= #TFF 0;
     end
    else
     begin
      //SPI3_RENB_n pin is active low.  When the SPI3_RENB_n pin is active the
      //Physical Layer does the appropriate operations, otherwise it
      //holds its previous state
      if (SPI3_RENB_n==0)
       begin
        //When the packet transfer is complete on the current
        //channel, do the upcoming transfer on the next channel.
        //When the transfer on the highest channel is complete, do
        //the next transfer on channel 0.
        if (word_num == LastWord)
         begin
          if (chan_num == TopChannel)
           begin
            chan_num <= #TFF 0;
           end
          else
           begin
            chan_num <= #TFF chan_num + 1;
           end
         end // if (word_num == LastWord)
       end // if (SPI3_RENB_n=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
  
 /******************************************************************************
  * The following always block Handles the word_num counter
  *****************************************************************************/

  integer i;
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      word_num <= #TFF AddressWord;
     end
    else
     begin
      //SPI3_RENB_n pin is active low.  When the SPI3_RENB_n pin is active the
      //Physical Layer does the appropriate operations, otherwise it
      //holds its previous state
      if (SPI3_RENB_n==0)
       begin
        //When the packet transfer is complete, reset word_num to
        //send an in-band address in the next cycle.  Otherwise,
        //move to the next word of the packet transfer
        if (word_num == LastWord)
         begin
          word_num <= #TFF AddressWord;
         end
        else
          begin
             word_num <= #TFF word_num + 1;
         end
       end // if (SPI3_RENB_n=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
 /******************************************************************************
  * The following always block Handles the delay_num counter
  *****************************************************************************/
  always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      delay_num <= #TFF 0;
     end
    else
     begin
      //SPI3_RENB_n pin is active low.  When the SPI3_RENB_n pin is active the
      //Physical Layer does the appropriate operations, otherwise it
      //holds its previous state
      if (SPI3_RENB_n==0)
       begin
        //delay_num performs 2 tasks.  It will cause RERR to go high
        //every n cycles (Where n is equal to DelayLength+1).  It
        //also is used to create a RMOD signal that cycles through
        //all of its possible values.
        if (delay_num == DelayLength)
         begin
          delay_num <= #TFF 0;
         end
        else
         begin
          delay_num <= #TFF delay_num + 1;
         end
       end // if (SPI3_RENB_n=='b0)
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
  
 /******************************************************************************
  * The following always block Handles the trans_num counter
  *****************************************************************************/
  always @ (posedge SPI3_TFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      trans_num <= #TFF 0;
     end
    else
     begin
      //Create a counter that counts to 50.  This is used with the TPA
      //signals to allow the eben numbered physical side FIFOs to be
      //full for a period of time and accept data for the same period of
      //time
      if (trans_num == 50)
       begin
        trans_num <= #TFF 0;
       end 
      else
       begin
        trans_num <= #TFF trans_num + 1;
       end
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
 /******************************************************************************
  * The following always block Handles the SPI3_PTPA Signal
  *****************************************************************************/
  always @ (posedge SPI3_TFClk or negedge SPI3_EMUL_RST_n)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      ptpa_i <= #TFF 0;
      ptpa_d0 <= #TFF 0;
     end
    else
     begin
      // The PTPA signal has a 1 cycle delay before being output.
      ptpa_i <= #TFF ptpa_d0;
 
      // Set the value of the PTPA signal
      //
      // The even channels will alternate between having full FIFOS
      // and being able to accept more data.
      //
      // Odd channels will always be able to accept more data.
      if ((SPI3_TADR % 2) == 0)
       begin
        if (trans_num >= 25)
         begin
          ptpa_d0 <= #TFF 0;
         end
        else
         begin
          ptpa_d0 <= #TFF 1;
         end
       end
      else
       begin
        ptpa_d0 <= #TFF 1;
       end
     end // else: !if (SPI3_EMUL_RST_n == 'b0)
   end // always @ (posedge SPI3_RFClk or negedge SPI3_EMUL_RST_n)
 
 /******************************************************************************
  * The following always block calculates the parity over the SPI3_RDAT bus
  *****************************************************************************/
  
  always @ (rdat_i or SPI3_EMUL_RST_n or rval_i or rsx_i)
   begin
    if (SPI3_EMUL_RST_n == 0)
     begin
      rprty_i <= 0;
     end
    else
     begin
      //The RPRTY signal is only valid when an in-band address is on
      //the RDAT bus (RSX is active) or when the Physical Layer is
      //transferring valid data (RVAL is active).
      if (rval_i == 1 || rsx_i == 1)
       begin
        //RPRTY will be low if there is an odd number of 1's on the
        //RDAT bus. This is to ensure odd parity
        rprty_i = ~(^rdat_i);
       end
     end // else: !ifSPI3_EMUL_RST_n
 
   end // always @ (rdat_i, SPI3_EMUL_RST_n)
  
 /******************************************************************************
  * Connect the internal signals to their respective output Ports
  *****************************************************************************/
 
  assign   SPI3_PTPA  = ptpa_i;
  
  assign   SPI3_RDAT  = rdat_i;
  assign   SPI3_RMOD  = rmod_i;
  assign   SPI3_RPRTY = rprty_i;
  assign   SPI3_RVAL  = rval_i;
  assign   SPI3_RSOP  = rsop_i;
  assign   SPI3_REOP  = reop_i;
  assign   SPI3_RERR  = rerr_i;
  assign   SPI3_RSX   = rsx_i;
 
endmodule
