-------------------------------------------------------------------------------
--
--   File Name:  spi3_emulator_phy.vhd
--     Version:  1.0
--        Date:  April 11, 2003
--
-- Description:  This file simulates a single-channel POS-PHY Level 3 Physical
--               Layer. It continuously transmits an eight word packet with
--               the appropriate Start Of Packet and End Of Packet Signals to
--               different channels.
--
--      Author:  IP Solutions Division
--     Company:  Xilinx Inc.
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- Library Declarations
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-------------------------------------------------------------------------------
-- Entity Declaration
-------------------------------------------------------------------------------
entity spi3_emulator_phy is

  -----------------------------------------------------------------------------
  -- Input and Output Declarations
  -----------------------------------------------------------------------------
  generic (
    DataOffset      : integer := 0
    );
  port (
    SPI3_emul_rst_n : in std_logic;

    SPI3_RFClk      : in std_logic;
    SPI3_REnb_n     : in std_logic;
    SPI3_RDat       : out std_logic_vector(31 downto 0);
    SPI3_RSOP       : out std_logic;
    SPI3_REOP       : out std_logic;
    SPI3_RMod       : out std_logic_vector(1 downto 0);
    SPI3_RPrty      : out std_logic;
    SPI3_RErr       : out std_logic;
    SPI3_RVal       : out std_logic;
    SPI3_RSx        : out std_logic;

    SPI3_TFClk      : in std_logic;
    SPI3_TEnb_n     : in std_logic;
    SPI3_TDat       : in std_logic_vector(31 downto 0);
    SPI3_TSOP       : in std_logic;
    SPI3_TEOP       : in std_logic;
    SPI3_TMod       : in std_logic_vector(1 downto 0);
    SPI3_TPrty      : in std_logic;
    SPI3_TErr       : in std_logic;
    SPI3_TSx        : in std_logic;
    SPI3_PTPA       : out std_logic;
    SPI3_TAdr       : in std_logic_vector(7 downto 0)
    );
end spi3_emulator_phy;

-------------------------------------------------------------------------------
-- Architecture Definition
-------------------------------------------------------------------------------
architecture behavioral of spi3_emulator_phy is

  -----------------------------------------------------------------------------
  -- Constant Declarations
  -----------------------------------------------------------------------------

  -- There are n channels that are numbered from 0 to n-1.  This constant
  -- represents the channel with the largest number. 
  constant TopChannel : integer := 0;   -- Only have a single channel core.

  -- NumOfWords is the number of 32 bit words that are transferred per packet.
  -- Modify this constant to change the number of words per packet.
  constant NumOfWords : integer := 8;

  -- AddressWord is the cycle where RSX is asserted and the in-band address is
  -- on the SPI3_RDat bus.
  constant AddressWord : integer := DataOffset;

  -- FirstWord is the cycle where SPI3_RSOP is asserted, it is the first word
  -- of the packet
  constant FirstWord : integer := 1;

  -- LastWord is the cycle where SPI3_REOP is asserted, it is the last word of
  -- the packet
  constant LastWord : integer := AddressWord + NumOfWords;

  -- DelayLength is used to limit the SPI3_RErr signal so that it does not
  -- occur in every cycle
  constant DelayLength : integer := 4;

  -- Constant time delay to model the clock to Q delay of registers.
  constant TFF : time := 2 ns;

  -----------------------------------------------------------------------------
  -- Signal Declarations
  -----------------------------------------------------------------------------

  signal ptpa_i  : std_logic                               := '0';
  signal ptpa_d0 : std_logic                               := '0';

  signal rdat_i  : std_logic_vector(31 downto 0);
  signal rmod_i  : std_logic_vector(1 downto 0 );
  signal rprty_i : std_logic;
  signal rval_i  : std_logic;
  signal rsop_i  : std_logic;
  signal reop_i  : std_logic;
  signal rerr_i  : std_logic;
  signal rsx_i   : std_logic;

  signal word_num  : integer range AddressWord TO LastWord := AddressWord;
  signal chan_num  : integer range 0 to TopChannel         := 0;
  signal delay_num : integer range 0 to DelayLength;
  signal trans_num : integer range 0 to 50;
  signal tchan_num : integer range 0 to TopChannel         := 0;

-------------------------------------------------------------------------------
-- Architecture Begin
-------------------------------------------------------------------------------

begin

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_RSOP Signal
-------------------------------------------------------------------------------
  rsop_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      rsop_i   <= '0' after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- When the Physical Layer is ready to transfer the first word of the
          -- packet, the Physical Layer will send out the SPI3_RSOP pulse to
          -- tell the Link layer that a data packet is beginning to transfer.
          if (word_num = FirstWord) then
            rsop_i <= '1' after TFF;
          else
            rsop_i <= '0' after TFF;
          end if;
        end if;
      end if;
    end if;
  end process rsop_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_REOP Signal
-------------------------------------------------------------------------------
  reop_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      reop_i   <= '0' after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- When the Physical Layer is ready to transfer the last word of the
          -- packet, the Physical Layer will send out the SPI3_REOP pulse to
          -- tell the Link layer that a data packet transfer is complete.
          if (word_num = LastWord) then
            reop_i <= '1' after TFF;
          else
            reop_i <= '0' after TFF;
          end if;
        end if;
      end if;
    end if;
  end process reop_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_RMod Signal
-------------------------------------------------------------------------------
  rmod_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      rmod_i   <= (others => '0') after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- Specify the number of bytes that are valid when in the last cycle
          -- of the packet transfer.  Otherwise set SPI3_RMod to all zeros. 
          -- Will cycle through all possible values ('00' to '11').
          if (word_num = LastWord) then
            rmod_i <= conv_std_logic_vector(delay_num mod 4, 2) after TFF;
          else
            rmod_i <= conv_std_logic_vector(0, 2) after TFF;
          end if;
        end if;
      end if;
    end if;
  end process rmod_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_RErr Signal
-------------------------------------------------------------------------------
  rerr_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      rerr_i   <= '0' after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then
          -- Specify that an error occured. This signal is activated on every
          -- nth packet (Where n is equal to Delay Length+1).
          -- The SPI3_RErr signal is only valid for the last cycle of the
          -- transfer. In all other cases SPI3_RErr is 0.
          if (word_num = LastWord) then
            if (delay_num = DelayLength) then
              rerr_i <= '1' after TFF;
            else
              rerr_i <= '0' after TFF;
            end if;
          else
            rerr_i <= '0' after TFF;
          end if;
        end if;
      end if;
    end if;
  end process rerr_proc;
  
-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_RDat Signal
-------------------------------------------------------------------------------
  rdat_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      rdat_i   <= (others => '0') after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- In the case that the Physical Layer is sending the in-band address
          -- on the SPI3_RDat bus, the format of the in-band address is as
          -- follows:
          --
          -- Bits 31-8: Ingored.  In this model they are all set to 0.
          --
          -- Bits 7-0: Contains the 8 bit encoding of the channel number that
          -- will be sending the next packet.
          if (word_num = AddressWord) then
            rdat_i <= conv_std_logic_vector(0, 8) &
                      conv_std_logic_vector(0, 8) &
                      conv_std_logic_vector(0, 8) &
                      conv_std_logic_vector(chan_num, 8) after TFF;

            -- In the case that the Physical Layer is sending valid data, the
            -- data is formatted as follows:
            --  
            -- Bits 31-24 : HEX coding for the channel number for which the
            -- transfer is taking place (ex. 01 is channel 1, 02 is channel 2)
            --  
            -- Bits 23-16 : Concatination of the current word that is being
            -- transferred and the byte number (byte number is always 1 for
            -- this position)
            --  
            -- Bits 15-8 : Concatination of the current word that is being
            -- transferred and the byte number (byte number is always 2 for
            -- this position)
            --  
            -- Bits 7-0 : Concatination of the current word that is being
            -- transferred and the byte number (byte number is always 3 for
            -- this position)
            --  
            -- (ex 01414243 is word 4 being transferred on channel 1)
          else
            rdat_i <= conv_std_logic_vector(chan_num, 8) &
                      conv_std_logic_vector((word_num*16)+1, 8) &
                      conv_std_logic_vector((word_num*16)+2, 8) &
                      conv_std_logic_vector((word_num*16)+3, 8) after TFF;
          end if;
        end if;
      end if;
    end if;
  end process rdat_proc;   

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_RVal Signal
-------------------------------------------------------------------------------
  rval_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      rval_i   <= '0' after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- When in the state where the in-band address will be put on the
          -- SPI3_RDat bus, the SPI3_RVal signal must be low so the SPI3_RSx
          -- signal will be valid.
          if (word_num = AddressWord) then
            rval_i <= '0' after TFF;
          else
            rval_i <= '1' after TFF;        
          end if;
        end if;
      end if;
    end if;
  end process rval_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_RSx Signal
-------------------------------------------------------------------------------
  rsx_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      rsx_i    <= '0' after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- When in the state where the in-band address will be put on the
          -- SPI3_RDat bus, the Physical device will activate the SPI3_RSx
          -- signal.
          if (word_num = AddressWord) then
            rsx_i  <= '1' after TFF;
          else
            rsx_i  <= '0' after TFF;
          end if;
        end if;
      end if;
    end if;
  end process rsx_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the chan_num counter
-------------------------------------------------------------------------------
  chan_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      chan_num <= 0 after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- When the packet transfer is complete on the current channel, do
          -- the upcoming transfer on the next channel.  When the transfer on
          -- the highest channel is complete, do the next transfer on channel 0
          if (word_num = LastWord) then
            if (chan_num = TopChannel) then
              chan_num <= 0 after TFF;
            else
              chan_num <= chan_num + 1 after TFF;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process chan_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the word_num counter
-------------------------------------------------------------------------------
  word_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      word_num <= AddressWord after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then          
          -- When the packet transfer is complete, reset word_num to send an
          -- in-band address in the next cycle.  Otherwise, move to the next
          -- word of the packet transfer
          if (word_num = LastWord) then
            word_num <= AddressWord after TFF;
          else
            word_num <= word_num + 1 after TFF;
          end if;
        end if;
      end if;
    end if;
  end process word_proc;    

-------------------------------------------------------------------------------
--  The Following Process Handles the delay_num counter
-------------------------------------------------------------------------------
  delay_proc : process (SPI3_RFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      delay_num <= 0 after TFF;
    else
      if (SPI3_RFClk = '1' and SPI3_RFClk'event) then
        -- SPI3_REnb_n pin is active low.  When the SPI3_REnb_n pin is active
        -- the Physical Layer does the appropriate operations, otherwise it
        -- holds its previous state
        if (SPI3_REnb_n = '0') then
          -- delay_num performs 2 tasks.  It will cause SPI3_RErr to go high
          -- every n cycles (Where n is equal to DelayLength+1).  It also is
          -- used to create a SPI3_RMod signal that cycles through all
          -- possible values.
          if (delay_num = DelayLength) then
            delay_num <= 0 after TFF;
          else
            delay_num <= delay_num + 1 after TFF;
          end if;
        end if;
      end if;
    end if;
  end process delay_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the trans_num counter
-------------------------------------------------------------------------------
  trans_proc : process (SPI3_TFClk, SPI3_emul_rst_n)
  begin
    if (SPI3_emul_rst_n = '0') then
      trans_num <= 0 after TFF;
    else
      if (SPI3_TFClk = '1' and SPI3_TFClk'event) then
        -- Create a counter that counts to 50.  This is used with the TPA
        -- signals to allow the even numbered physical side FIFOs to be full
        -- for a period of time and accept data for the same period of time.
        if (trans_num = 50) then
          trans_num <= 0 after TFF;
        else
          trans_num <= trans_num + 1 after TFF;
        end if;
      end if;
    end if;
  end process trans_proc;

-------------------------------------------------------------------------------
--  The Following Process Handles the SPI3_PTPA Signal
-------------------------------------------------------------------------------
  ptpa_proc : process (SPI3_TFClk, SPI3_emul_rst_n)
    -- Create a variable that will hold the integer value of SPI3_TAdr so that
    -- calculations can be done using its value.
    VARIABLE tadr_int : integer := 0;
  begin
    if (SPI3_emul_rst_n = '0') then
      ptpa_i  <= '0' after TFF;
      ptpa_d0 <= '0' after TFF;
    else
      if (SPI3_TFClk = '1' and SPI3_TFClk'event) then
        -- Convert SPI3_TAdr to an integer value for calculations below
        tadr_int := conv_integer(SPI3_TAdr);
        
        -- The SPI3_PTPA signal has a 1 cycle delay before being output.   
        ptpa_i <= ptpa_d0 after TFF;
        
        -- Set the value of the SPI3_PTPA signal
        --
        -- The even channels will alternate between having full FIFOS
        -- and being able to accept more data.
        --
        -- Odd channels will always be able to accept more data.          
        if ((tadr_int mod 2) = 0) then
          if (trans_num >= 25) then
            ptpa_d0 <= '0' after TFF;
          else
            ptpa_d0 <= '1' after TFF;
          end if;
        else
          ptpa_d0 <= '1' after TFF;
        end if;               
      end if;
    end if;
  end process ptpa_proc;    


-------------------------------------------------------------------------------
--  The following process calculates the parity over the SPI3_RDat bus
-------------------------------------------------------------------------------
  parity : process (rdat_i, SPI3_emul_rst_n, rval_i, rsx_i)
  begin
    if SPI3_emul_rst_n = '0' then
      rprty_i <= '0';
    else

      -- The SPI3_RPrty signal is only valid when an in-band address is on the
      -- SPI3_RDat bus (SPI3_RSx is active) or when the Physical Layer is
      -- transferring valid data (SPI3_RVal is active).
      if rval_i = '1' or rsx_i = '1' then

        -- SPI3_RPrty will be low if there is an odd number of 1's on the
        -- SPI3_RDat bus. This is to ensure odd parity
        rprty_i <= not ( xor_reduce(rdat_i));

      end if;
    end if;
  end process parity;




-------------------------------------------------------------------------------
-- Connect the internal signals to their respective output Ports
-------------------------------------------------------------------------------
  SPI3_PTPA  <= ptpa_i;

  SPI3_RDat  <= rdat_i;
  SPI3_RMod  <= rmod_i;
  SPI3_RPrty <= rprty_i;
  SPI3_RVal  <= rval_i;
  SPI3_RSOP  <= rsop_i;
  SPI3_REOP  <= reop_i;
  SPI3_RErr  <= rerr_i;
  SPI3_RSx   <= rsx_i;

end behavioral;

